*! version 0.4.1 2012-09-04 jsl | dcp ocp work | posted
* version 0.4.0 2012-09-03 jsl | code clean | posted
    * drop coeftypes for std; coeftypespace to stdlabelspace; drop saving()

program define _ordcplot_syntax, sclass

    version 11.2

*! DO: use global till final constant chosen; then hard code
    global ordc_type_spaces = 6 // use global till final value chosen

*TRACE di in blue _new "====> Entering _ordcplot_syntax"

    syntax [varlist(default=none)], [ /// O = orplot  D = dcplot
///
/// axis and graph region
    max(real 99999) /// OD max of x axis
    min(real -99999) /// OD min of x axis
    gap(real 0) /// OD gap between tick marks
    NTics(integer 5) /// OD # of tics on axis; or use gap()
    XLINEs(string) /// OD xline options (size color everything)
///
/// plot region and labeling
    ASPECTratio(real 0) /// OD aspect ratio for graph
    DECimals(integer 2) /// OD dec digits on axis labels
    ORDECimals(integer 2) /// O  dec digits for the OR scale
///
/// symbols for outcome categories
    BASEcategory(integer -1) /// O  base category for OR plot
    DChange /// O  add DC to or plot
    MColors(string) /// OD color of markers
    MSIZefactor(string) /// OD size of markers; factor only
    nosign /// O  do not underline negative DC's in orplot
    std(string) /// OD Binary Unstd Sstd Rrange  b u s r
    VALues /// OD plot values not letters of value labels
///
/// connecting list
    LINEPvalue(string) /// LINEPvalue(real .1) /// O  line if < less sig
    LColors(string) /// O  colors for connecting lines
    LINEGAPfactor(real 1) /// O  factor change gap around symbols for lines
    LWidths(string) /// O  width of connecting lines
    PACKed /// O  no offset of markers
    OFFSETLIST(string) /// list of 1 per category per variable ranging from 5 to -5
                       /// in order var1 cat1 cat2 cat3... var2 cat1 cat2... etc
///
/// labels and notes
    CAPtion(string) /// OD text for provenance
    Note(string) /// OD text with options for graph note() option
    PROVenance(string) /// caption with formatting (No OPTprovenance)
    SUBtitle(string) /// OD text with options for subtitle() option
    TItle(string) /// OD text with options for graph title() option
    VARLabels /// OD use variable labels to label variables
///
/// adjusting defaults
    GRAPHoptions(string) /// OD pass through options to plot commands; TO TEST
    OFFSETFactor(real 1) /// O  multiply vertical offsets in OR plot by this
    OFFSETSEQuence(string) /// O  0 1 2 is default for vertical offsets
    STDLABELSPaces(integer 0) /// OD # spaces at end of coef type label; can be <0
///
/// utility and to be added
    debug(integer 0) /// OD
    matrix /// OD enter data as matrices not estimation
    ]

//  syntax and error checking

    if "`offsetsequence'"!="" & "`offsetlist'"!="" {
        display as error ///
        "offsetlist() and offsetsequence() cannot be used together"
        sreturn local error = 1
        exit
    }

    if `min'>=`max' {
        display as error ///
        "the minimum is larger than the maximum"
        sreturn local error = 1
        exit
    }

    local std : subinstr local std " " "", all

    if "`linepvalue'"=="" local linepvalue = .1
    if "`linepvalue'"!="" {
        local tnosort : list retoken linepvalue
        local tnosort : list uniq linepvalue
        local linepvalue : list sort tnosort
        local issorted = 0
        if "`linepvalue'"!="`tnosort'" {
            local issorted = 1
            display ///
            "Note: options sorted to linepvalue(`tsort')"
        }
        local npvalue = wordcount("`linepvalue'")
        if `npvalue'>5 {
            display as error ///
            "linepvalue() can only have 5 values psecified"
            sreturn local error = 1
            exit
            contine
        }
        foreach p in `linepvalue' {
            capture confirm number `p'
            if _rc {
                display as error ///
                "linepvalue() must contain numbers; `p' is invalid"
                sreturn local error = 1
                exit
                contine
            }
        }
        if "`lcolor'"=="" local lcolor black
        if "`lwidth'"=="" local lwidth thin
        local xlcolor "`lcolor' `lcolor' `lcolor' `lcolor' `lcolor'"
        local xlwidth "`lwidth' `lwidth' `lwidth' `lwidth' `lwidth'"
        local lcolor ""
        local lwidth ""
        forvalues i = 1(1)`npvalue' {
            local s : word `i' of `xlcolor'
            local lcolor "`lcolor' `s'"
            local s : word `i' of `xlwidth'
            local lwidth "`lwidth' `s'"
        }
    }

    if "`caption'"!="" & "`provenance'"!="" {
        display as error ///
        "caption() and provenance() cannot both be used"
        sreturn local error = 1
        exit
    }

    if "`matrix'" == "matrix" {
        if `linepvalue'!=.1 {
            display as error ///
            "DO: add linepvalue() features for matrix input."
            local linepvalue = .1
        }
        if "`dchange'"=="dchange" & "`_ordc[_plot_type_]'"== "oratio" {
            display as error ///
            "DO: add dchange to matrix input"
            local dchange ""
        }
    }

    if "`provenance'"!="" {
        local provopts " size(tiny)" // size(vsmall)"
        local caption "Source: `provenance', `provopts'"
        local provenance ""
    }

    if "`e(cmd)'"=="mlogit" {
        local plottype "or"
    }
    * supported commands for dcplot
    else if "`e(cmd)'"=="oprobit" | "`e(cmd)'"=="slogit" ///
        | "`e(cmd)'"=="ologit" {
        local plottype "dc"
    }
    else {
        display as error ///
        "`_ordc[_plot_type_]' does not work with `e(cmd)'"
        sreturn local error = 1
        exit
    }
    if "`_ordc[_plot_type_]'"=="orplot" & "`plottype'"=="dc" {
        display as error ///
        "`_ordc[_plot_type_]' cannot be run with `e(cmd)'"
        sreturn local error = 1
        exit
    }

//  characteristics with all options for dcp and orp

    capture qui gen _orp = .
    label var _orp "Variable holds orplot command as characteristics"
    capture qui gen _dcp = .
    label var _dcp "Variable holds dcplot command as characteristics"


    #delimit ;
    * options using ()'s ;
    local optparlist "
        xlines
        title
        subtitle
        stdlabelspaces
        std
        provenance
        ordecimals
        offsetsequence
        offsetlist
        offsetfactor
        ntics
        note
        msizefactor
        min
        mcolors
        max
        lwidths
        linepvalue
        linegapfactor
        lcolors
        graphoptions
        gap
        decimals
        debug
        caption
        basecategory
        aspectratio
    ";

    * options using only words ;
    local optparword "
        varlabels
        values
        packed
        nosign
        matrix
        dchange
    ";
    #delimit cr

    if ("`_ordc[_plot_type_]'"=="orplot") local charvar "_orp"
    if ("`_ordc[_plot_type_]'"=="dcplot") local charvar "_dcp"

    foreach opt in `optparlist' `optparword' {
        local optsave = ltrim("``opt''")
        if "``opt''"=="" local optsave "_none_"
        char `charvar'[`opt'] "`optsave'"
    }

//  globals with submitted orplot or dcplot command
//      - to use with orp and dcp commands

    #delimit ;
    * options using ()'s ;
    local optparlist "
        aspectratio     basecategory    caption         std
        stdlabelspaces  debug           decimals        gap
        graphoptions    lcolors         linegapfactor   linepvalue
        lwidths         mcolors         max             min
        msizefactor     note            ntics           offsetfactor
        offsetsequence  offsetlist      ordecimals      saving
        title           subtitle        xlines
    ";

    * options using only words ;
    local optparword "
        dchange         matrix          nosign          packed
        values          varlabels
    ";
    #delimit cr

    local _optP ""
    local _optW ""
    foreach opt in `optparlist' {
        if (`"``opt''"'!="") local _optP `"`_optP'`opt'(``opt'') "'
    }
    foreach opt in `optwordlist' {
        if (`"``opt''"'!="") local _optW "`_optW'`opt' "
    }
    if "`_ordc[_plot_type_]'"=="orplot" local ordc "or"
    if "`_ordc[_plot_type_]'"=="dcplot" local ordc "dc"

    global `ordc'plot_cmdname  "`ordc'plot"
    global `ordc'plot_varlist  "`varlist' "
    global `ordc'plot_optP     `"`_optP'"'
    global `ordc'plot_optW     "`_optW'"

    if "`ordc'"=="or" {
        global orplot_cmd `"orplot $orplot_varlist,"'
        global orplot_cmd `"$orplot_cmd $orplot_optW $orplot_optP"'
    }
    if "`ordc'"=="dc" {
        global dcplot_cmd `"dcplot $dcplot_varlist,"'
        global dcplot_cmd `"$dcplot_cmd $dcplot_optW $dcplot_optP"'
    }

// which variables to plot

    local OPTvarnms `varlist'
    if "`OPTvarnms'"=="" & "`OPTmatrix'"!="matrix" {
        _rm_rhsnames OPTvarnms rhsN1 rhsNMS2 rhsN2
    }

//  set up plotting options

    if ("`_ordc[_plot_type_]'"=="dcplot") local packed packed
    local OPTmaxvars = 7
    if ("`packed'"=="packed") local OPTmaxvars = 11

    //  default std() if user did not set them
    if "`std'"=="" {
        foreach varnm in `OPTvarnms' {
            rm3_pedum `varnm'
            if (`r(dummy)'==1) local std "`std'b"
            else local std "`std's"
        }
    }

//  information for args in other _ordcplot programs

    local debug = `debug'
    local OPTaspect = `aspectratio'
    local OPTbasecat = `basecategory'
    local OPTcaption `"`caption'"'
    local OPTstd "`std'"
    local OPTstdlabelspaces = `stdlabelspaces'
    local OPTdchange "`dchange'"
    local OPTdcplot "`dcplot'"
    local OPTdecimals = `decimals'
    local OPTdepvar "`depvar'"
    local OPTgap = `gap'
    local OPTgraph `"`graphoptions'"'
    local OPTlcolor `"`lcolors'"'
    local OPTlinegapfac = `linegapfactor'
    local OPTlinepvalue "`linepvalue'"
    local OPTlwidth "`lwidths'"
    local OPTmatrix "`matrix'"
    local OPTmax = `max'
    local OPTmaxvars "`OPTmaxvars'"
    local OPTmcolor `mcolors'
    local OPTmin = `min'
    local OPTmsize `msizefactor'
    local OPTnosign "`nosign'"
    local OPTnote `"`note'"'
    local OPTntics = `ntics'
    local OPToffsetfac = `offsetfactor'
    local OPToffsetseq "`offsetsequence'"
    local OPToffsetlist "`offsetlist'"
    local OPTordec = `ordecimals'
    local OPTpacked "`packed'"
    local OPTrefnm "`refnm'"
    local OPTsaving "`saving'"
    local OPTtitle `"`title'"'
    local OPTsubtitle `"`subtitle'"'
    local OPTvalues "`values'"
    local OPTvarlabels "`varlabels'"
    local OPTvarnms "`OPTvarnms'"
    local OPTxlines "`xlines'"

    #delimit ;
    local OPTall `"
    "`debug'"               "`OPTaspect'"           "`OPTbasecat'"
    "`OPTcaption'"          "`OPTstd'"              "`OPTstdlabelspaces'"
    "`OPTdchange'"          "`OPTdcplot'"           "`OPTdecimals'"
    "`OPTdepvar'"           "`OPTgap'"              "`OPTgraph'"
    "`OPTlcolor'"           "`OPTlinegapfac'"       "`OPTlinepvalue'"
    "`OPTlwidth'"           "`OPTmatrix'"           "`OPTmax'"
    "`OPTmaxvars'"          "`OPTmcolor'"           "`OPTmin'"
    "`OPTmodelok'"          "`OPTmsize'"            "`OPTnosign'"
    "`OPTnote'"             "`OPTntics'"            "`OPToffsetfac'"
    "`OPToffsetseq'"        "`OPToffsetlist'"       "`OPTordec'"
    "`OPTpacked'"           "`OPTrefnm'"            "`OPTsaving'"
    "`OPTtitle'"            "`OPTsubtitle'"         "`OPTvalues'"
    "`OPTvarlabels'"        "`OPTvarnms'"           "`OPTxlines'"
    "'
    ;
    #delimit cr

//  get data to be plotted

    _ordcplot_data `OPTall'

*TRACE di in blue _new "    = 3 data  => Back from _ordcplot_data"

    if `s(error)'==1 {
        exit
        sreturn local error = 1
    }

//  submit graph command

    _ordcplot_graph `OPTall'

*TRACE di in blue _new "    = 3 graph => Back from _ordcplot_graph"

    if `s(error)'==1 {
        exit
        sreturn local error = 1
    }

//  closeup

    capture drop _P_*
    sreturn local error = 0

*TRACE di in blue _new "<==== Leaving _ordcplot_syntax"

end
exit

HISTORY

* version 0.0.1
* version 3.0.3a 2012-08-02 10.01 major refactoring
* version 0.3.5 2012-08-03 scott long
* version 0.3.6c 2012-08-06 offsetlist
* version 0.3.6b 2012-08-06 multiple line colors
* version 0.3.6a 2012-08-06 title2 to subtitle
* version 0.3.7 working version
* version 0.3.8 2012-09-03 jsl | work on nom chapter | posted
    * std() instead of coeftypes()
