*! version 3.0.3 removed redundant removal of base equation
*! version 3.0.2 2012-07-31 remove omitted

//  reshape e(b) and e(V) to square matrix like Stata 5
//  o. coefficients are dropped.

capture program drop rm3_get_mlogit_bv
capture program drop rm3_remove_baseeq
program rm3_get_mlogit_bv
    version 9
    args b v
    if `"`b'"'=="" | `"`v'"'=="" {
        di as err "rm3_get_mlogit_bv: names for b and v must be specified"
        exit 198
    }
    if `"`e(cmd)'"'!="mlogit" & `"`e(cmd)'"'!="mprobit" { // v301 add mprobit
        di as err "rm3_get_mlogit_bv: model not mlogit or mprobit"
        exit 498
    }

    // get copy of e(b) and e(V)
    matrix `b' = e(b)
    matrix `v' = e(V)

    // remove omitted variables v302
    tempname noomit
    local coln : colfullnames `b'
    local cols = colsof(`b')
    _ms_omit_info `b' // names of o. columns
    matrix `noomit' = J(1,`cols',1) - r(omit)
    * take columns out of matrices
    mata: noomit = st_matrix(st_local("noomit"))
    mata: newb = select(st_matrix(st_local("b")), noomit)
    mata: st_matrix(st_local("b"),newb)
    mata: newv = select(select(st_matrix(st_local("v")), noomit), noomit')
    mata: st_matrix(st_local("v"),newv)
    * reassign column names to b
    foreach var of local coln {
        _ms_parse_parts `var'
        if (!`r(omit)') local coln2 `coln2' `var'
    }
    matrix colnames `b' = `coln2'
    matrix colnames `v' = `coln2'
    matrix rownames `v' = `coln2'

    // remove base eq
*V303    rm3_remove_baseeq `b' `v'

    // reshape b to (ncat-1) rows by (nvars + 1) columns
    local eqs: coleq `b', quoted
    local eqs: list uniq eqs
    tempname tmp bnew
    local r 0
    foreach eq of local eqs {
        local ++r
        mat `tmp' = `b'[1, `"`eq':"']
        mat rown `tmp' = y`r'
        mat `bnew' = nullmat(`bnew') \ `tmp'
    }
    * mat coleq `bnew' = : // v302
    mat coleq `bnew' = "" // v302
    mat drop `b'
    mat rename `bnew' `b'

end

/*
program rm3_remove_baseeq
    args b v
    if `"`b'"'=="" | `"`v'"'=="" {
        di as err "_remove_baseeq: b and v must be specified"
        exit 198
    }
    if c(stata_version) < 11    exit        // Stata 11 (or newer) only
    if `"`e(cmd)'"'!="mlogit" & `"`e(cmd)'"'!="mprobit" {
        exit
    }
    local ibase = e(k_eq_base)              // get base equation number
    capt confirm integer number `ibase'     // check validity of ibase
    if _rc exit
    if `ibase'>=. | `ibase'<1   exit
    _ms_eq_info, matrix(`b')                // get equations info
    local l = 0                             // determine subscripts to
    forv i = 1/`r(k_eq)' {                  // remove base equation:
        if `i' == `ibase' continue, break   //   l = last element before
        local l = `l' + r(k`i')             //       base eq, or 0
    }                                       //   r = first element after
    local i = `l'                           //       base eq, or .
    while (`++i' <= r(k`ibase')) {          // make sure that base eq is,
        if `b'[1,`i']!=0        exit        // in fact, a base eq (all 0)
    }
    local r = cond(`ibase' >= r(k_eq), ., `l' + r(k`ibase') + 1)
di "DOG5"
    if `l' > 0 & `r' < . {                  // base eq within
        mat `b' = `b'[1..., 1..`l'] , `b'[1..., `r'...]
        mat `v' = `v'[1..., 1..`l'] , `v'[1..., `r'...]
        mat `v' = `v'[1..`l', 1...] \ `v'[`r'..., 1...]
    }
    else if `r' < . {                       // base eq at beginning
        mat `b' = `b'[1..., `r'...]
        mat `v' = `v'[1..., `r'...]
        mat `v' = `v'[`r'..., 1...]
    }
    else if `l' > 0 {                       // base eq at end
        mat `b' = `b'[1..., 1..`l']
        mat `v' = `v'[1..., 1..`l']
        mat `v' = `v'[1..`l', 1...]
    }
end
*/
exit

HISTORY

* version 3.0.1 2012-07-25
* version 3.0.0 2012-06-02 08.11
* version 1.0.2 07oct2009 Ben Jann new reshape algorithm
* version 1.0.1 15sep2009 jsl
* version 1.0.0 15sep2009 Ben Jann
