
*! version 3.0.2 2012-08-07 allow svy

* compute summary statistics for RHS variables from last model estimated

capture program drop rm3_pesum
program define rm3_pesum, rclass

    version 11.2

    tempname Smean Ssd Smin Smax Sdummy Smedian SN m v

    syntax [fweight aweight pweight] [if] [in] ///
        [ , Median Dummy Two ]

    //  svy setup

    local issvy = 0
    if ("`e(prefix)'"=="svy") local issvy = 1
    
    if `issvy' {
        
        * since svy : mean is e command, save e results
        tempname model
        estimates store `model'
        di "stored"
        
    }

    //  weights: check if wts used with the  estimation command. 

    local wtis "[`weight'`exp']"

    if "`wtis'"=="" & "`e(wtype)'"!="" {

        local weight "`e(wtype)'"
        local wtis "[`e(wtype)'`e(wexp)']"

    }

    if "`weight'"=="pweight" {

        local wtis "[aweight`e(wexp)']"
        di "note: pweights are treated as aweights to" ///
            " compute standard deviations"

    }

    if "`weight'"=="iweight" {

        di as error "iweights cannot be used for this command"
        exit

    }

    //  names of variables

    rm3_perhs

*di "rm3_perhs"    
*return list
*ereturn list

    local nvars = `r(nrhs)' + 1
    local varnms "`e(depvar)' `r(rhsnms)'"

    * 2nd eq for zip and zinb
    if "`two'"=="two" {

        local nvars = `r(nrhs2)' + 1
        local varnms "`e(depvar)' `r(rhsnms2)'"

    }

    * intreg has two lhs vars; select the first
    if "`e(cmd)'"=="intreg" {

        local nmtoget : word 1 of `varnms'
        local varnms "`nmtoget' `r(rhsnms)'"

    }

    //  compute statistics

    matrix `Smean'   = J(1,`nvars',-99999)
    matrix colnames    `Smean' = `varnms'
    matrix `Ssd'     = `Smean'
    matrix `Smin'    = `Smean'
    matrix `Smax'    = `Smean'
    matrix `Sdummy'  = `Smean'
    matrix `Smedian' = `Smean'

    local ivar = 1
    while `ivar'<=`nvars' {

        local nmtoget : word `ivar' of `varnms'

        qui sum `nmtoget' `wtis' `if' `in'

        scalar `SN' = r(N)

        if `SN' == 0 {
            return scalar SN = `SN'
            di as error "estimation sample is 0"
            exit
            continue
        }

        matrix `Smin'[1,`ivar']  = r(minn)
        matrix `Smax'[1,`ivar']  = r(max)
        
        if `issvy' {

*di ". svy : mean  `nmtoget' `if' `in'"

            qui capture svy : mean  `nmtoget' `if' `in'

            if _rc!= 0 {
                di as error "cannot compute svy : mean required for results"
                di as error "note that svy : mean does not work with factor variables"
                exit _rc
                continue
            }

            qui estat sd                

            matrix `m' = r(mean)
            scalar `v' = `m'[1,1]
            matrix `Smean'[1,`ivar'] = `v'

            matrix `m' = r(sd)
            scalar `v' = `m'[1,1]
            matrix `Ssd'[1,`ivar'] = `v'
            
        }
        
        else { // not svy

            matrix `Smean'[1,`ivar'] = r(mean)
            matrix `Ssd'[1,`ivar']   = sqrt(r(Var))

        }

        * no weights needed for determing if is dummy
        if "`dummy'"=="dummy" {
            rm3_pedum `nmtoget' `if' `in'
            matrix `Sdummy'[1,`ivar'] = r(dummy)
        }

        if "`median'"=="median" {
            
            if `issvy' {
                di as error "the median cannot be computed with svy"
                exit
                continue
            }
            
            _ms_parse_parts `nmtoget'

            if "`r(type)'" != "variable" {
                qui summarize `nmtoget', detail
                matrix `Smedian'[1,`ivar'] = r(p50)
            }

            else {
                qui _pctile `nmtoget' `if' `in' `wtis'
                matrix `Smedian'[1,`ivar'] = r(r1)
            }

        } // median

        local ++ivar

    } // loop over variables

    //  return results 
    
    return matrix Smean   `Smean'
    return matrix Ssd     `Ssd'
    return matrix Smin    `Smin'
    return matrix Smax    `Smax'
    return matrix Sdummy  `Sdummy'
    return matrix Smedian `Smedian'
    return scalar SN = `SN'

    //  restore model estimates

    if `issvy' {

        estimates restore `model'
        
    }

end

exit

HISTORY

* version 1.6.3 05/25/2009
* version 2.0.0 07jul2009 factor variables allowed
* version 3.0.0 2012-06-02 initial RM3 versions
* version 3.0.1 2012-07-30 ommit o. variables

NOTES

2012-08-07 
    svy : mean does not work with factor variables
    
2012-08-07
    can't compute svy medians
    