program drop _all
capture log close _all
log using rm3ch4brm-v3, replace text

//  program:    rm3ch4brm-v3.do
//  task:       chapter 4 - brm
//  project:    rm3
//  versions:   v3 spost12 with non-numbered m commands
//              v2 spost12 using fvs to reproduce results without fv's
//              v1 spost12 commands but not fv's
//              v0 replication against spost9
//  author:     jsl 2012-08-28

version 12.1
clear all
macro drop _all
set linesize 80
set scheme sj

local exstub "rm3ch4v3"
local pgm "rm3ch4brm-v2fv"
local dte "2012-08-28"
local tag "`pgm'.do jsl `dte'"

program drop _all
include spost12_which.doi

//  #1 estimation using logit and probit

sysuse binlfp4, clear
describe lfp k5 k618 age wc hc lwg inc
summarize lfp k5 k618 age wc hc lwg inc

logit lfp c.k5 k618 age wc hc lwg inc, nolog
estimates store logit
probit lfp c.k5 k618 age wc hc lwg inc, nolog
estimates store probit

estimates table logit probit, b(%9.3f) t label varwidth(30)

//  #2 predicting perfectly (using artificial data) (pg 118 RevEd)
//  see Allison on perfect prediction

sysuse science4, clear
gen college = 1 - mmale
gen vote = pub1>10
tab vote college
logit vote college phd, nolog

//  #3 testing individual coefficients (pg 119 RevEd)

sysuse binlfp4, clear
* fv: do not need c.
logit lfp k5 k618 age i.wc i.hc lwg inc, nolog vsquish
* fv: can use c.
logit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog vsquish

* Wald test
test c.k5
* fv: does not need c.
test k5
di sqrt(55.14)
di sqrt(r(chi2))

//  #4 testing multiple coefficients (pg 122 RevEd)

* LR test
logit lfp k5 k618 age i.wc i.hc lwg inc, nolog vsquish
estimates store fmodel
logit lfp k618 age i.wc i.hc lwg inc, nolog vsquish
estimates store nmodel
lrtest fmodel nmodel
lrtest fmodel . // . refers to latest model


* Wald tests
* fv test i.hc i.wc    error
* fv test hc wc        error
test 1.hc 1.wc
test 1.hc=1.wc
* fv: test age or test c.age are fine
test age
test c.age

* LR tests
logit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog vsquish
estimates store fmodel
logit lfp c.k5 c.k618 c.age c.lwg c.inc, nolog vsquish
estimates store nmodel
lrtest fmodel nmodel

logit lfp, nolog vsquish
estimates store intercept_only
lrtest fmodel intercept_only

//  #5 graph of residuals and influence used in text (pg 124 RevEd)

clear
set seed 315
set obs 34
gen y = (runiform()*4) in 1/30
label var y "Simulated y"
gen x = (4 - (y)) + (runiform()) in 1/30
label var x "Simulated x related to y with outlier"

* change one point and compute the regression
replace y = 9 in 31
summarize x
scalar meanx = r(mean)
replace x = meanx in 31
regress y x

* change two points
replace x = 0 in 32
replace x = 5 in 33
predict yhat in 32/33
label var yhat "Predicted y"
predict res in 32/33, res
label var res "Residual from y on x"

*change one value of y and compute predictions from the original model
set textsize 150
drop yhat res
predict yhat
label var yhat "Predicted y"
predict res , res
label var res "Residual from y on x"
local yout = y[31]
local xout = x[31]

* create artificial data
clear
set seed 315
set obs 34
gen y = (runiform()*4) in 1/30
gen x = (4 - (y)) + (runiform()) in 1/30
* change one point
replace y = 9 in 31
summarize x
scalar meanx = r(mean)
replace x = meanx in 31
* compute the regression
regress y x
* change two points
replace x = 0 in 32
replace x = 5 in 33
* compute predictions
predict yhat in 32/33
predict res in 32/33, res
gen y3 = 9 in 31
set textsize 150
drop yhat res
predict yhat
predict res , res
* save these values for the plot
local yout = y[31]
local xout = x[31]

* graph outlier that is not influential
graph twoway ///
    (scatter y x, msymbol(dh) mcolor(black)) ///
    (line yhat x, sort) ///
    (scatteri `yout' `xout' (3) "outlier"), ///
        title("Large outlier that is not influential") ///
        legend(pos(6) order( 2 ) label(2 "Regression line") ) ///
        ytitle("y") xlabel( 0 1 2 3 4 5 ) name(reg1, replace)

* change the data and run another regression
replace y = 7 in 31
replace x = 29.5 in 31
replace x = 30 in 34
local yout = y[31]
local xout = x[31]
regress y x
predict yhat2
replace yhat2 = . in 2/31

* graph the results
graph twoway ///
    (scatter y x, msymbol(dh) mcolor(black)) ///
    (line yhat2 x, sort clpattern(dash)) ///
    (line yhat x if x<25, sort) ///
    (scatteri `yout' `xout' (9) "influential observation  ", ///
        msymbol(S) mlabgap(*3)), ///
        ylabel(0 2 4 6 8 10) xlabel( 0 10 20 30) ///
        title("Smaller outlier that is influential") ///
        legend(pos(6) order( 2 3) rows(2) ///
            label(2 "With influential observation") ///
            label(3 "Without influential observation") ) ///
        ytitle("y") name(reg2,replace)

*  combine the two graphs
graph combine reg1 reg2, col(1) ysize(6.24) xsize(4.784) iscale(*.9)
graph export `exstub'04residstata.emf, replace

//  #6 residuals

sysuse binlfp4, clear
logit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog vsquish
predict rstd, rs
label var rstd "Standardized Residual"
* fv: sort c.inc, stable does not work
sort inc, stable
generate index = _n
label var index "Observation Number"

* index plot of std pearson residuals
graph twoway scatter rstd index, xlabel(0(200)800) ylabel(-4(2)4) ///
    xtitle("Observation Number") yline(0) msymbol(Oh)
graph export `exstub'04rstd.emf , replace

* index plot of std pearson residuals labeled with the index
graph twoway scatter rstd index, xlabel(0(200)800) ylabel(-4(2)4) ///
    xtitle("Observation Number") yline(0) ///
    msymbol(none) mlabel(index) mlabposition(0)
graph export `exstub'04rstdcase.emf , replace

* index plot of std pearson residuals for large values
* fv: mlabel(c.k5) causes and error
graph twoway scatter rstd index if (rstd>1.7) | (rstd<-1.7), ///
    msymbol(none) mlabel(k5) mlabposition(0) ///
    caption("Values indicate # of young children") ///
    xlabel(0(200)800) xtitle("Observation Number") ///
    ylabel(-4(2)4) yline(0)
graph export `exstub'04rstdbig.emf , replace

* list a single point
list in 142, noobs

* list large outliers
list rstd index if rstd>2.5 | rstd<-2.5

//  #7 influential cases (pg 128 RevEd)

predict cook, dbeta
label var cook "Cook's Statistic"
graph twoway scatter cook index, xlabel(0(200)800) ylabel(0(.1).3) ///
    xtitle("Observation Number") yline(.1 .2) ///
    msymbol(none) mlabel(index) mlabposition(0)
graph export `exstub'04cookcase.emf , replace

//  #8 least likely cases (pg  2Ed)

sysuse binlfp4, clear
logit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog vsquish
rleastlikely k5 k618 1.wc

//  #9 scalar measures of fit (pg 129 RevEd)

quietly logit lfp k5 k618 age wc hc lwg inc, nolog vsquish
estimates store model1
quietly rfitstat, save // matches fitstat 2012-08-24 08.58
rfitstat, save

gen agesq = age*age
logit lfp k5 age agesq wc inc, nolog vsquish
estimates store model2
rfitstat, diff // matches fitstat 2012-08-24 08.58

estimates table model1 model2, b(%9.3f) t

quietly logit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog vsquish
estimates store model1
quietly rfitstat, save // matches fitstat 2012-08-24 08.58
rfitstat, save

capture drop agesq
* fv: cannot use: gen c.agesq = c.age*c.age  can't gen c....
gen agesq = c.age*c.age
logit lfp c.k5 c.age c.agesq i.wc c.inc, nolog vsquish
estimates store model2
estimates table model1 model2, b(%9.3f) t
rfitstat, diff // matches fitstat 2012-08-24 08.58

* using fv for age squared
logit lfp c.k5 c.age##c.age i.wc c.inc, nolog vsquish
rfitstat

//  #10 hosmer-lemeshow test

logit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog vsquish
estat gof, group(10)
estat gof

//  #11 lowess plot

predict p1
label var p1 "Pr(lfp|X)"
lowess lfp p1, ylabel(0(.2)1, grid) xlabel(0(.2)1, grid) ///
    addplot(function y = x, legend(off))
graph export `exstub'04hllowess.emf, replace

//  #12 predicted probabilities with predict

sysuse binlfp4, clear
logit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog vsquish
predict prlogit
summarize prlogit
label var prlogit "Logit: Pr(lfp)"
dotplot prlogit, ylabel(0(.2)1)
graph export `exstub'04dotpredict.emf, replace

* comparing logit and probit predictions
sysuse binlfp4, clear
logit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog vsquish
predict prlogit
label var prlogit "Logit: Pr(lfp)"
probit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog vsquish
predict prprobit
label var prprobit "Probit: Pr(lfp)"
pwcorr prlogit prprobit

* graphing predicted probabilities from logit and probit
graph twoway scatter prlogit prprobit, ///
    xlabel(0(.25)1) ylabel(0(.25)1) ///
    xline(.25(.25)1) yline(.25(.25)1) ///
    plotregion(margin(zero)) msymbol(Oh)
graph export `exstub'04logitprobit.emf, replace

//  #13 individual predicted probabilities

sysuse binlfp4, clear
probit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog vsquish
predict prprobit
label var prprobit "Probit: Pr(lfp)"

* fv: margins, at(c.age=35 c.k5=2 i.wc=0 i.hc=0 c.inc=15) atmeans
*       i:  operator invalid
* fv: margins, at(c.age=35 c.k5=2 1.wc=0 1.hc=0 c.inc=15) atmeans
*   at values for factor wc do not sum to 1

* young, low c.income, low education families with young children.
margins, at(age=35 k5=2 wc=0 hc=0 inc=15) atmeans
mtable, at(age=35 k5=2 wc=0 hc=0 inc=15) atmeans mlistat

* highly educated families with no children at home.
margins, at(age=50 k5=0 k618=0 wc=1 hc=1) atmeans
mtable, at(age=50 k5=0 k618=0 wc=1 hc=1) atmeans mlistat

* an average person
margins, atmeans
mtable, atmeans mlistat

//  #14 tables of predicted probabilities

* fv: margins, atmeans at(i.wc=0 k5=(0 1 2 3))
* fv: i:  operator invalid
* fv: margins, atmeans at(1.wc=0 c.k5=(0 1 2 3))
* fv: at values for factor wc do not sum to 1

margins, atmeans at(wc=0 c.k5=(0 1 2 3))
mtable, atmeans at(wc=0 c.k5=(0 1 2 3)) predname(PrLFP)

*! must rerun margins due to bug in Stata on margins _return
margins, atmeans at(wc=(0 1) c.k5=(0 1 2 3))
mtable,  atmeans at(wc=(0 1) c.k5=(0 1 2 3)) atvars(i.wc c.k5)
mtable,  atmeans at(wc=(0 1) c.k5=(0 1 2 3)) atvars(1.wc c.k5)
* NOTE:  {space -1} appears to be a bug in matlist
mtable,  atmeans at(wc=(0 1) c.k5=(0 1 2 3)) atvars(1.wc k5)

*! must rerun margins due to bug in Stata on margins _return
margins, atmeans at(wc=(0 1) c.k5=(0 1 2 3))
mtable, atmeans at(wc=(0 1) c.k5=(0 1 2 3)) atvars(1.wc k5)

* RM3note: changing at() does not change output order
margins, atmeans at(wc=(0 1) c.k5=(0 1 2 3))
margins, atmeans at(wc=(0 1) c.k5=(0 1 2 3))
margins, atmeans at(c.k5=(0 1 2 3) wc=(0 1))
* RM3note: multiple at()'s does change order
margins, atmeans at(c.k5=(0 1 2 3) wc=(0)) ///
                 at(c.k5=(0 1 2 3) wc=(1))
mtable, atmeans at(c.k5=(0 1 2 3) wc=(0)) at(c.k5=(0 1 2 3) wc=(1))

estimates store _prepost
margins, atmeans at(c.k5=(0 1 2 3) wc=(0 1)) post
* compute the differences:
di 0.6069-0.7758
di 0.2633-0.4449
di 0.0764-0.1565
di 0.0188-0.0412

    //  RM3do: have mlincom create tables; collect results
matrix list e(b)
mlincom, exp(1-2)
mlincom, exp(3-4)
mlincom, exp(5-6)
mlincom, exp(7-8)

estimates restore _prepost
margins, atmeans dydx(wc) at(k5=(0 1 2 3))
mtable,  atmeans dydx(wc) at(k5=(0 1 2 3)) title(Change from wc=0 to wc=1)

* RM3note: using dydx to compute dc's
* RM3note: must restore since post
* RM3note: using post and mlincom for difference of difference
margins, atmeans at(k5=(0 1 2 3) wc=(0 1)) post
* is effect of wc for k5=0 same as for k5=1
mlincom , exp((1-2)-(3-4))
* is effect of wc for k5=0 same as for k5=2
mlincom , exp((1-2)-(5-6))
* is effect of wc for k5=1 same as for k5=2
mlincom , exp((3-4)-(5-6))

//  #15 graphing predicted probabilities

sysuse binlfp4, clear
probit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog vsquish

* compute predictions at age 30
mgen, atmeans at(c.inc=(0(10)100) age=30) stub(a30)
label var a30inc "Income"
label var a30p  "Pr(LFP|age=30,x)"
nmlab a30*

foreach yr in 40 50 60 {
    * compute predictions at age yr
    mgen, atmeans at(c.inc=(0(10)100) age=`yr') stub(a`yr')
    label var a`yr'p  "Pr(LFP|age=`yr',x)"
}

* list and graph predictions
nmlab a*
list a30p a40p a50p a60p a30inc in 1/11
        *sjlog close, replace

* list and graph predictions
label var a30p "Age 30"
label var a40p "Age 40"
label var a50p "Age 50"
label var a60p "Age 60"

list a30p a40p a50p a60p a30inc in 1/11
graph twoway connected a30p a40p a50p a60p a30inc, ///
    ytitle("Pr(In Labor Force)") ylabel(0(.25)1) xtitle("Income")
graph export `exstub'04ageincome.emf, replace

//  #16 plotting confidence intervals

mgen, atmeans at(c.age=(20(2)70)) stub(ci)
label var cip "Predicted probability"
label var ciub "95% upper limit"
label var cilb "95% lower limit"
label var ciage "Age"

* using lines to show confidence interval
twoway ///
    (connected cip ciage, ///
        clcolor(black) clpat(solid) clwidth(medthick) ///
        msymbol(i) mcolor(none)) ///
    (connected ciub ciage, ///
        msymbol(i) mcolor(none) ///
     clcolor(black) clpat(dash) clwidth(thin)) ///
    (connected cilb ciage, ///
         msymbol(i) mcolor(none) ///
         clcolor(black) clpat(dash) clwidth(thin)), ///
    ytitle("Probability of Being in Labor Force") yscale(range(0 .35)) ///
    ylabel(, grid glwidth(medium) glpattern(solid)) ///
    xscale(range(20 70)) xlabel(20(10)70)
graph export `exstub'04ageci.emf, replace

* using shading to show confidence interval
graph twoway ///
    (rarea cilb ciub ciage, color(gs14)) ///
    (connected cip ciage, ///
        clcolor(black) clpat(solid) clwidth(medthick) ///
        msymbol(i) mcolor(none)), ///
    ytitle("Probability of Being in Labor Force") yscale(range(0 .35)) ///
    ylabel(, grid glwidth(medium) glpattern(solid)) ///
    xscale(range(20 70)) xlabel(20(10)70) ///
    legend(label (1 "95% confidence interval"))
graph export `exstub'04ageci2.emf, replace

//  #17 compare those who do and do not attend college (not in book)

sysuse binlfp4, clear
logit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog vsquish

mgen, atmeans at(c.age=(20(2.5)60) wc=1) stub(wc1)
label var wc1p "Attended College"
label var wc1age "Age"
mgen, atmeans at(c.age=(20(2.5)60) wc=0) stub(wc0)
label var wc0p "Did Not Attend College"
graph twoway connected wc1p wc0p wc1age, ///
    xtitle("Age") ytitle("Pr(In Labor Force)") ///
    ylabel(0(.25)1) xlabel(30(10)60)
graph export `exstub'04agewc0wc1.emf, replace

//  #18 changes in predicted probabilities (pg 139 RevEd)

sysuse binlfp4, clear
logit lfp c.k5 c.k618 c.age i.wc i.hc c.lwg c.inc, nolog vsquish

* DC1 for age at age 40 wc=1
mchange c.age, atmeans at(wc=1 c.age=40) dec(4) change(_all)

* DC2 c.age or age works
mchange age, atmeans at(wc=1 c.age=40) dec(4) change(_all)

* DC3 at means for all variables
mchange , atmeans dec(4) change(_all)

* DC4 for k5 age wc lwc inc computing all types of changes with
*       dc, start value and end value of change
mchange c.k5 c.age i.wc c.lwg c.inc, ///
    atmeans dec(4) change(_all) stats(dc start end)

* DC5 all types of change for wc with CI
mchange i.wc, atmeans change(_all) brief

* DC6 01 changes for k5 k618 wc; to from dc ub and lb
mchange c.k5 c.k618 i.wc, atmeans stats(to from dc lb ub) brief change(01)

* DC7 centered sd change
mchange c.age c.lwg c.inc, atmeans stats(to from dc lb ub) brief change(sd)

* DC8 instead of prvalue, we get ci's and from to with mchange
mchange c.age, atmeans at(c.age=30) delta(10) uncentered ///
    stats(to from dc lb ub) change(sd) brief

//  #19 odds ratios using -listcoef- (pg 146 RevEd)

rlistcoef, help
rlistcoef, help reverse
rlistcoef, help percent

capture log close _all
exit