capture log close _all
log using rm3ch6nomcase-v3b, replace text

//  program:    rm3ch6nomcase-v2-part1.do
//  task:       chapter 6: nrm case based
//  project:    rm3
//  versions:   v3 spost12 with non-numbered m commands
//                  b using updated mtabe
//              v2 spost12 using fvs to reproduce results without fv's
//              v1 spost12 commands but not fv's
//              v0 replication against spost9
//  author:     scott 2012-09-15; 2012-09-18

//  Regression Models for Categorical Dependent Variables - 2nd Edition
//  Chapter 6 - Models for Nominal Outcomes for Case-based Data
//  Long and Freese - 11Aug2005

version 12.1
clear all
macro drop _all
set linesize 80
set scheme sj

local exstub "rm3ch6v3b-"
program drop _all
include spost12_which.doi

//  #1
//  intro to multinomial logit model

//  construct new outcome variables with 2 or 3 categories

sysuse nomocc4, clear
tab occ
gen occ3 = occ
recode occ3 1/3=1 4=2 5=3
label def occ3 1 Manual 2 WhiteCol 3 Prof
label val occ3 occ3
tab occ occ3,miss
gen wc_man   = occ3==2 if occ3!=3
gen prof_man = occ3==3 if occ3!=2
gen prof_wc  = occ3==3 if occ3!=1
label def wc_man   1 WhiteCol 0 Manual
label def prof_man 1 Prof 0 Manual
label def prof_wc  1 Prof 0 WhiteCol
label val wc_man    wc_man
label val prof_man  prof_man
label val prof_wc   prof_wc
tab occ3 wc_man, miss
tab occ3 prof_man, miss
tab occ3 prof_wc, miss
tab prof_man, miss
label data "nomintro.dta : temporary data set for mlogit example"
save nomintro4, replace

//  binary logits to inefficiently estimate mnlm

sysuse nomintro4, clear
tab prof_man, miss
logit prof_man ed, nolog
tab wc_man, miss
logit wc_man ed, nolog
tab prof_wc, miss
logit prof_wc ed, nolog

//  corresponding multinomial logit

tab occ3, miss
mlogit occ3 ed, nolog

//  #2
//  effect of different category values for outcome

gen occ999 = occ3
recode occ999 1=-3 2=0 3=999
mlogit occ3 ed, nolog b(3)
estimates store mnlmocc3
mlogit occ999 ed, nolog b(999)
estimates store mnlmocc999
estimates table mnlmocc3 mnlmocc999

//  #3
//  example of occupational attainment

sysuse nomocc4, clear
desc white ed exper
summarize white ed exper
tab occ
mlogit occ i.white ed exper, baseoutcome(5) nolog

//  using different base categories

rlistcoef i.white, help
rlistcoef, pvalue(.05)
rlistcoef ed, pvalue(.05) gt nolabel

//  #4
//  predicting perfectly (example not in book)

//  DO: check allison's article

version 10
sysuse ordwarm4, clear
gen byte dprst = (prst<20 & warm==1)
label var dprst "Prestige < 20 and warm==1"
tab dprst warm, miss
mlogit warm yr89 male white age ed dprst, nolog
estimates store perfectallcases
drop if dprst==1
mlogit warm yr89 male white age ed, nolog
estimates table perfectallcases .

version 12.1
sysuse ordwarm4, clear
gen byte dprst = (prst<20 & warm==1)
label var dprst "Prestige < 20 and warm==1"
tab dprst warm, miss
mlogit warm yr89 i.male i.white age ed dprst, nolog
estimates store perfectallcases
drop if dprst==1
mlogit warm yr89 i.male i.white age ed, nolog
estimates table perfectallcases .

//  #5
//  testing the effects of the independent variables (pg 20

//  LR tests of single coefficient using -lrtest-

sysuse nomocc4, clear
mlogit occ i.white ed exper, baseoutcome(5) nolog
estimates store fmodel

* test of white
quietly mlogit occ ed exper, baseoutcome(5) nolog
estimates store nmodel_white
lrtest fmodel nmodel_white

* test of ed
mlogit occ i.white exper, baseoutcome(5) nolog
estimates store nmodel_ed
lrtest fmodel nmodel_ed

*fv: i.white in one model; white in other is ok
mlogit occ i.white ed exper, baseoutcome(5) nolog
est store iwhite
mlogit occ white exper, baseoutcome(5) nolog
lrtest iwhite .

* test of exper
quietly mlogit occ i.white ed, baseoutcome(5) nolog
estimates store nmodel_exper
lrtest fmodel nmodel_exper

* tests using -mlogtest-
quietly mlogit occ i.white ed exper, baseoutcome(5) nolog
rmlogtest, lr

//  Wald tests of single variable using -test-

quietly mlogit occ i.white ed exper, baseoutcome(5) nolog

* using -test-
test ed
test exper
test 1.white
* RM3fv error: test white
* RM3fv error: test i.white

* using -mlogtest-
rmlogtest, wald

//  LR test of multiple variables

* using -lrtest-
mlogit occ i.white ed exper, baseoutcome(5) nolog
estimates store fmodel
mlogit occ i.white, baseoutcome(5) nolog
estimates store nmodel
lrtest fmodel nmodel

* using -mlogtest-
mlogit occ i.white ed exper, baseoutcome(5) nolog
rmlogtest, lr set(ed exper)

//  Wald test of multiple variables

* using -test-
mlogit occ i.white ed exper, baseoutcome(5) nolog
test ed exper

* using -mlogtest-
rmlogtest, wald set(ed exper)

//  #6
//  tests for combining dependent categories

//  Wald test of combining categories

* using -test-
mlogit occ i.white ed exper, baseoutcome(5) nolog

* using -mlogtest-
rmlogtest, combine
test [Menial]
test [Menial=Craft]

//  LR test of combining categories

* using -mlogtest-
mlogit occ i.white ed exper, baseoutcome(5) nolog
rmlogtest, lrcomb

* constraints imposed by -mlogtest-
mlogit occ i.white ed exper, nolog
estimates store fmodel
constraint define 999 [Menial]
mlogit occ exper ed i.white, base(2) constraint(999) nolog
estimates store nmodel
lrtest fmodel nmodel

//  #7
//  irrelevant alternatives

* Hausman test of iia
mlogit occ i.white ed exper, baseoutcome(5) nolog
rmlogtest, hausman base

* Hausman using alternative outcomes (not in book)
quietly mlogit occ i.white ed exper, baseoutcome(1) nolog
rmlogtest, hausman base
quietly mlogit occ i.white ed exper, baseoutcome(2) nolog
rmlogtest, hausman base
quietly mlogit occ i.white ed exper, baseoutcome(3) nolog
rmlogtest, hausman base
quietly mlogit occ i.white ed exper, baseoutcome(4) nolog
rmlogtest, hausman base

* Small-Hsiao test of iia
quietly mlogit occ i.white ed exper, baseoutcome(5) nolog
set seed 339487731
rmlogtest, smhsiao
set seed 8675309
rmlogtest, smhsiao

//  #8
//  in sample predicted probabilities with -predict-

sysuse nomocc4, clear

* estimate model and compute predictions
mlogit occ i.white ed exper, baseoutcome(5) nolog
predict ProbM ProbB ProbC ProbW ProbP
describe Prob*
summarize Prob*

* -predict- to compare -mlogit- and -ologit-
sysuse ordwarm4,clear
ologit warm yr89 male i.white age ed prst, nolog
predict SDologit Dologit Aologit SAologit
label var Dologit "ologit-D"
mlogit warm yr89 male i.white age ed prst, nolog
predict SDmlogit Dmlogit Amlogit SAmlogit
label var Dmlogit "mlogit-D"
dotplot Dologit Dmlogit, ylabel(0(.25).75)
graph export `exstub'06omlogdot.emf, replace

* an interesting way to compare the predictions (not in book)
corr SDologit SDmlogit
corr Dologit Dmlogit
corr Aologit Amlogit
corr SAologit SAmlogit
graph twoway scatter Dologit Dmlogit, ///
    xtitle("mlogit: Pr(y=Disagree)") ///
    ytitle("ologit: Pr(y=Disagree)") ///
    ylabel(0(.25).75) xlabel(0(.25).75) sort
graph export `exstub'06omloggraph.emf, replace

//  #9
//  individual predicted probabilities with -margins-

sysuse nomocc4, clear
mlogit occ i.white ed exper, baseoutcome(5) nolog

mtable, atmeans at(white=1) valuelen(4) mlistat
* BUG: need to run margins again due to stata _restore bug
mtable, at(*) valuelen(4) mlistat

//  #10
//  tables of predicted probabilities

sysuse nomocc4, clear
mlogit occ i.white ed exper, baseoutcome(5) nolog

* RM3note: levelsof gets values of ed for predictions
levelsof ed if e(sample), local(edvals)
* this takes a long time due to margins! use verbose to watch progress
mtable , at(ed=(`edvals') white=(0 1)) atmeans atvars(ed 1.white) ///
    verbose
* without labels
mtable , at(ed=(`edvals') white=(0 1)) atmeans atvars(ed 1.white) nolabel ///
    verbose

//  #11
//  graphing predicted probabilities with -mgen-

sysuse nomocc4, clear

* plotting probabilities to compare two groups
quietly mlogit occ white ed exper, baseoutcome(5)
mgen, atmeans at(ed=(6(1)20) white=1) stub(rmW)
describe rmW*

mgen, atmeans at(ed=(6(1)20) white=0) stub(rmN)
describe rmN*

label var rmWp1 "Whites"
label var rmNp1 "Nonwhites"
label var rmWed "Education"

* p1 for the two groups
graph twoway connected rmWp1 rmNp1 rmWed, ///
    xtitle("Years of Education") ytitle("Pr(Menial Job)") ///
    ylabel(0(.25).50) xlabel(6 8 12 16 20)
graph export `exstub'06prmenial.emf, replace

* probabilities for all outcomes for one group
label var rmWCp1 "Menial"
label var rmWCp2 "Blue Collar"
label var rmWCp3 "Craft"
label var rmWCp4 "White Collar"
graph twoway connected rmWCp1 rmWCp2 rmWCp3 rmWp4 rmWed, ///
    xtitle("Whites: Years of Education") ytitle("Summed Probability")                        ///
    xlabel(6(2)20) ylabel(0(.25)1)
graph export `exstub'06prsum.emf, replace

* shaded cumulative graphs
capture gen zero = 0
capture gen one  = 1
graph twoway (rarea zero rmWCp1 rmWed, bc(gs1)) ///
    (rarea rmWCp1 rmWCp2 rmWed, bc(gs4)) ///
    (rarea rmWCp2 rmWCp3 rmWed, bc(gs8)) ///
    (rarea rmWCp3 rmWCp4 rmWed, bc(gs11)) ///
    (rarea rmWCp4 one rmWed, bc(gs14)), ///
    ytitle("Summed Probability") legend( order( 1 2 3 4 5) ///
    label( 1 "Menial") label( 2 "Blue Collar") label( 3 "Craft") ///
    label(4 "White Collar") label(5 "Professional")) ///
    xtitle("Whites: Years of Education") ///
    xlabel(6 8 12 16 20) ylabel(0(.25)1) ///
    plotregion(margin(zero))
graph export `exstub'06prsumrevised.emf, replace

//  #12
//  changes in predicted probabilities

sysuse nomocc4, clear
mlogit occ i.white ed exper, baseoutcome(5) nolog
mchange, atmeans

mlogit occ i.white ed exper, baseoutcome(2) nolog
mchange, atmeans

//  #13
//  DC plots

sysuse nomocc4, clear
mlogit occ i.white ed exper, baseoutcome(5) nolog
mchange, atmeans

//  DC at means: dcp to perfect plot
dcplot, std(buu) ntics(9) aspect(.5) mcolors(rainbow)
dcp, aspect(.2)
dcp, varlabels mcol(rainbow)
graph export `exstub'DC_1a_2atmean.emf,replace

//  DC at means: using other options
dcplot, std(bss) min(-.25) max(.5) gap(.05) aspect(.5)
dcp, mcol(rainbow) aspect(.3) subtitle(Discrete change at mean, position(11))
graph export `exstub'DC_1b_atmean.emf,replace

//  DC mean of:
mchange
dcplot, std(bss) min(-.25) max(.5) gap(.05) ///
    mcol(rainbow) aspect(.3) subtitle(Mean of discrete change, position(11))
graph export `exstub'DC_2a_meanof.emf,replace

//  #14
//  OR plots

sysuse nomocc4, clear
mlogit occ i.white ed exper,baseoutcome(5) nolog
rlistcoef 1.white, help

//  OR w/o sig levels
* RM3note: fv if i.white, error, must be 1.
orplot 1.white ed exper, std(bss) base(5) linep(1) min(-2.75) max(.55) ///
    packed ntics(7) aspect(.5)
orp, aspect(.5) mcolors(rainbow) varlabel
graph export `exstub'OR_1a_nosig.emf,replace

//  OR with significance levels
orplot 1.white ed exper, linep(.1) std(bss) base(5) ///
    min(-2.75) max(.55) ntics(7) aspect(.5)
orp, mcol(rainbow) linegapfactor(.9)
graph export `exstub'OR_1b_sig.emf, replace

//  OR with two levels of significance
orp, linep(.1 .2) lcolor(gs12 gs5) ///
    subtitle(Dark line p>.2; light line p>.1, position(11))
graph export `exstub'OR_1c_2levels.emf, replace

//  #14
//  OR + DC plots

mchange, atmeans
orplot 1.white ed exper, linep(.1) std(bss) base(5) ///
    min(-2.75) max(.55) dc ntics(7) aspect(.5) mcolors(rainbow)
orp, linegapfac(.7) // bug not showing dc?
graph export `exstub'ORDC_1a.emf, replace

//  adjusting the offsets
orplot 1.white ed exper, dc std(bss) linep(.1) ///
    min(-2.75) max(.55)  ntics(7) mcolor(rainbow) linegap(.8)
orp, offsetlist(0 3 0 -3 0   2 -2 -2 0 0   1 1 -3 0 3 )
graph export `exstub'ORDC_1b.emf, replace

//  #15
//  plotting estimates from matrices with mlogplot (pg 231 RevEd)

global eg1lhsnm   "Y"
global eg1catnms  "B C A"
global eg1catvals "1 2 3"

global eg1basecat = 1
* betas vs base:     x1b   x2b   x3b    x1c    x2c   x3c
matrix eg1beta  = (-.693, .693, .347 \ .347, -.347, .693 )
* Sig contrasts:    1-2 1-3 2-3;  next variables
matrix eg1pval  = ( .1, .01, .3,   .2 , .3, .01,   .01, .1, .04 )

global eg1rhsnms "x1 x2 x3"
global eg1x1     "Variable 1"
global eg1x2     "Variable 2"
global eg1x3     "Variable 3"
matrix eg1sd    = (1, 1, 1) // sd
matrix eg1dummy = (0, 0, 0) // is it dummy
matrix eg1range = (1, 1, 1) // range
orplot,  matrixstub(eg1) std(uuu) packed varlabel aspect(.4)
graph export `exstub'ORmatrix_1a.emf, replace

matrix eg1dc = ( ///                          Discrete Changes
                 .5 , -.2 , -.3 \         /// rng     x1
                 .5 , -.2 , -.3 \         /// 01
                 .5 , -.2 , -.3 \         /// 1
                 .5 , -.2 , -.3 \         /// sd
                 .5 , -.2 , -.3 \         /// partial
                 .5 , -.2 , -.3 \         /// rng     x2
                 .5 , -.2 , -.3 \         /// 01
                 .5 , -.2 , -.3 \         /// 1
                 .5 , -.2 , -.3 \         /// sd
                 .5 , -.2 , -.3 \         /// partial
                 .5 , -.2 , -.3 \         /// rng     x3
                 .5 , -.2 , -.3 \         /// 01
                 .5 , -.2 , -.3 \         /// 1
                 .5 , -.2 , -.3 \         /// sd
                 .5 , -.2 , -.3           /// partial
)
orplot,  matrixstub(eg1) std(uuu) packed varlabel aspect(.4) dc
graph export `exstub'ORmatrix_1b.emf, replace

//  #16
//  combining coefficients from two groups

* estimate two sets of coefficients
sysuse nomocc4, clear
mlogit occ ed exper if white==1, base(5) nolog
rlistcoef ed

mlogit occ ed exper if white==0, base(5) nolog
rlistcoef ed

* compute sd of ed and add to matrix
summarize ed

global eg2lhsnm   "occ"
global eg2catnms  "Menial BlueCol Craft WhiteCol Prof"
global eg2catvals "1 2 3 4 5"
global eg2basecat = 5

* betas
matrix eg2beta  = (-.8307514, -.9225522, -.6876114, -.4196403 \   ///
                  -.7012628, -.560695 , -.882502 , -.5311514 )
matrix eg2beta = eg2beta'
* Sig contrast
matrix eg2pval  =  ( ///
 .430 , .214 , .002 , .000 , .006 , .000 , .000 , .008 , .000 , .000 , ///
 .591 , .421 , .592 , .035 , .207 , .925 , .088 , .258 , .009 , .151 )
global eg2rhsnms "White NonWhite"
global eg2x1     "White-Educ"
global eg2x2     "NonWhite-Educ"
matrix eg2sd = (2.946427,2.946427)
matrix eg2dummy = (0, 0) // is it dummy
matrix eg2range = (1, 1) // range
global eg2cmd "mlogit"
orplot,  matrixstub(eg2) std(ss) varlabel aspect(.4) ///packed
            subtitle("Racial Differences in Effects of Education")
            *   m b c   w p   m b c   w p
orp, offsetlist(0 3 -3  0 0   3 3 -3 -1 -1 ) min(-3) max(1.2)
graph export `exstub'ORmatrix_2a.emf, replace

//  #17
//  multinomial probit (pg  2Ed)

* compare mprobit coefficients to probit coefficients
sysuse binlfp2, clear
probit lfp k5 k618 age i.wc i.hc lwg inc, nolog
mprobit lfp k5 k618 age i.wc i.hc lwg inc, nolog baseoutcome(0)
mprobit lfp k5 k618 age i.wc i.hc lwg inc, nolog base(0) probitparam

* compare mprobit and probit predictions
sysuse binlfp2, clear
probit lfp k5 k618 age i.wc i.hc lwg inc, nolog
predict p_probit1
mprobit lfp k5 k618 age i.wc i.hc lwg inc, nolog base(0)
predict p_mprobit0 p_mprobit1
mprobit lfp k5 k618 age i.wc i.hc lwg inc, nolog base(0) probitparam
predict p_mprobit0p p_mprobit1p
pwcorr p_probit1 p_mprobit1 p_mprobit0 p_mprobit1p p_mprobit0p

* compare mprobit and mlogit
sysuse nomocc4, clear
mprobit occ i.white ed exper, base(1)
predict mpp1 mpp2 mpp3 mpp4 mpp5
* with probit normalization
mprobit occ i.white ed exper, base(1) probitparam
mlogit occ i.white ed exper, base(1)
predict mlp1 mlp2 mlp3 mlp4 mlp5
correlate *p1
gen prdif1 = abs(mpp1-mlp1)
gen prdif2 = abs(mpp2-mlp2)
gen prdif3 = abs(mpp3-mlp3)
gen prdif4 = abs(mpp4-mlp4)
gen prdif5 = abs(mpp5-mlp5)
summarize prdif*

* dcplot for mprobit
mprobit occ i.white ed exper, base(1) probitparam
mchange
dcplot
graph export `exstub'DCmprobit_1a.emf, replace

//  #18
//  stereotype logit model (pg  2Ed)

sysuse ordwarm4, replace
tab warm
tab warm, nolabel

* mlogit for attitudes toward working mothers
mlogit warm yr89 male i.white age ed prst, nolog baseoutcome(4)

* slm for attitudes toward working mothers
slogit warm yr89 male i.white age ed prst, nolog

* dcplot
mchange
dcplot, mcolor(blue ltblue pink red)
graph export `exstub'DCslogit_1a.emf, replace

* predictions for the estimation sample
predict slpr1 slpr2 slpr3 slpr4
list warm slpr1-slpr4 in 1/4

* slogit compared to mlogit
mlogit warm yr89 male i.white age ed prst, nolog base(4)
predict mlpr1 mlpr2 mlpr3 mlpr4
list warm slpr1 mlpr1 slpr2 mlpr2 slpr3 mlpr3 in 1/5
pwcorr slpr1 mlpr1
pwcorr slpr2 mlpr2
pwcorr slpr3 mlpr3
pwcorr slpr4 mlpr4

* predictions
qui slogit warm yr89 male i.white age ed prst, nolog
estimates store _slogit
margins, atmeans at(male=(0 1)) post
mlincom , exp(1-2)
estimates restore _slogit

mchange male, atmeans

* odds ratios
rlistcoef, expand

* distinguishability and the phi parameters
slogit warm yr89 male i.white age ed prst, nolog
est store slbase

* wald test that phi1=phi2
nlcom ([phi1_1]_b[_cons] - [phi1_2]_b[_cons])
* phi1=phi2
constraint define 1 [phi1_1]_cons=[phi1_2]_cons

slogit warm yr89 male i.white age ed prst, nolog constraint(1)
lrtest slbase .

* phi2=phi3
constraint define 2 [phi1_2]_cons=[phi1_3]_cons
slogit warm yr89 male i.white age ed prst, nolog constraint(2)
lrtest slbase .

* phi3=phi4
constraint define 2 [phi1_3]_cons=0
slogit warm yr89 male i.white age ed prst, nolog constraint(2)
lrtest slbase .

* phi1=phi2 and phi3=phi4
slogit warm yr89 male i.white age ed prst, nolog constraint(1 2)
lrtest slbase .

* ordinality and the 1-dimensional slogit model
gen warm2 = warm
recode warm2 1=1 2=3 3=2 4=4
slogit warm2 yr89 male i.white age ed prst, nolog
est store warm2
slogit warm yr89 male i.white age ed prst, nolog
est store warm
estimates table warm warm2, b(%9.3f) t(%6.2f)

* dimensions in slogit
slogit warm yr89 male i.white age ed prst, nolog
slogit warm yr89 male i.white age ed prst, nolog dim(2)

capture log close
exit
