program drop _all
capture log close _all
log using rm3ch8count-v3b, replace text

//  program:    rm3ch8count-v3b.do
//  task:       chapter 4 - brm
//  project:    rm3
//  versions:   v0 replication against spost9
//              v3 2012-09-19 with spost12; v3b updated mtable
//
//  author:     jsl 2012-09-19

version 12.1
clear all
macro drop _all
set linesize 80
set scheme sj

local exstub "rm3ch8v3"
program drop _all
*include spost12_which.doi
sysuse couart4, clear

//  Regression Models for Categorical Dependent Variablesn - 2nd Edition
//  Chapter 8 - Models for Count Outcomes
//  Long and Freese - 27Jul2005

//  #1
//  the poisson distribution p.350

clear
set obs 25
local n = 0
foreach v in .8 1.5 2.9 10.5 {
    local ++n
    gen y`n' = `v'
    qui poisson y`n', nolog
    qui mgen, out(0(1)20) plot stub(p`n') force
    label var p`n'preq "mu=`v'"
}

label var p1val "y=# of Events"
graph twoway connected p1preq p2preq p3preq p4preq p1val, ///
    ytitle("Probability") ylabel(0(.1).5) xlabel(0(2)20)
graph  export `exstub'08poisson.emf, replace

//  #2
//  fitting the poisson distribution with poisson 350

sysuse couart4, clear
describe
summarize
poisson art, nolog

//  #3
//  plotting the poisson distribution with mgen p.352/354

sysuse couart4, clear
poisson art, nolog
qui mgen, out(0(1)9) plot stub(psn)
label var psnobeq "Observed Proportion"
label var psnpreq "Poisson Prediction"
label var psnval "# of Articles"
list psnval psnobeq psnpreq in 1/10
graph twoway connected psnobeq psnpreq psnval, ///
    ytitle("Probability") ylabel(0(.1).4) xlabel(0(1)9)
graph export `exstub'08psnpred.emf, replace

//  #4
//  prm - estimation

* non-integer outcomes - note warning!
sysuse couart4, clear
gen artx = art + .3
poisson artx i.fem mar kid5 phd ment, nolog

* if and in conditions
poisson art mar kid5 phd ment if fem==1, nolog

* estimation with count outcome p.359
sysuse couart4, clear
poisson art i.fem i.mar c.kid5 phd ment, nolog

//  #5
//  prm - interpretation using the rate mu

* factor changes p.360
rlistcoef 1.fem ment, help

* percent change p.361
rlistcoef 1.fem ment, percent help

* marginal change with mchange p.362
poisson art i.fem mar kid5 phd ment, nolog
* ! the pvalues really are identical
mchange phd ment, atmeans change(partial)

* discrete change p.363/365
mchange 1.fem ment, atmeans
mchange kid5, atmeans at(kid5=1) uncentered stat(dc)
mchange kid5, atmeans at(kid5=1) uncentered stat(dc) delta(2)

//  #6
//  prm - interpretation using predicted probabilities p.365

poisson art i.fem i.mar kid5 phd ment, nolog

* mu p.366

* rate
mtable, at(mar=0 fem=1 kid5=0) at(mar=1 fem=1 kid5=0) atmeans
* probabilities; right option merges results with prior mu estimates
mtable, at(mar=0 fem=1 kid5=0) at(mar=1 fem=1 kid5=0) ///
    atmeans outcome(0(1)5) width(6) atoff right

* discrete change
mtable, dydx(mar) at(fem=1 kid5=0) atmeans

* effects of number of children p.366
mtable, at(mar=1 fem=1 kid5=(0(1)3)) atmeans // rate
mtable, at(mar=1 fem=1 kid5=(0(1)3)) atmeans ///
    outcome(0(1)4) atoff width(6) right

* mgen p.367
qui mgen, at(fem=1 mar=1 kid5=(0(1)3)) stub(fprm) outcome(0)
qui mgen, at(fem=0 mar=1 kid5=(0(1)3)) stub(mprm) outcome(0)
label var fprmp0 "Married Women"
label var mprmp0 "Married Men"
label var mprmkid5  "Number of Children"
graph twoway connected fprmp0 mprmp0 mprmkid5, ///
    ylabel(0(.1).4) yline(.1 .2 .3) xlabel(0(1)3) ///
    ytitle("Probability of No Articles")
graph export `exstub'08prmprob0.emf, replace

list fprmp0 mprmp0 mprmkid5 in 1/4

* predictions from univariate poisson distribution p.369
poisson art, nolog
qui mgen, atmeans outcome(0(1)9) stub(psn) plot
label var psnpreq "Univariate Poisson Dist."

* for prm p.369
poisson art fem mar kid5 phd ment, nolog
qui mgen, atmeans outcome(0(1)9) stub(prm) plot
label var prmpreq "PRM"
label var prmobeq "Observed"

* compare pdf to prm p.370
graph twoway connected prmobeq psnpreq prmpreq prmval, ///
    ytitle("Probability of Count") ylabel(0(.1).4) xlabel(0(1)9)
graph export `exstub'08prmpred.emf, replace

//  #7
//  prm - exposure time p.370

sysuse couart4, clear

* simulate data to illustrate exposure
* construct random integers
set seed 11197411
gen profage = round((1+10*uniform()),1)
label var profage "Professional Age"
gen lnage = ln(profage)
label var lnage "Log of Professional Age"
* let total articles equal artiles*exposure time
gen totalarts = art*profage
label var totalarts "Total Articles during Career"

* PRM without including time
poisson totalarts i.fem mar kid5 phd ment, nolog

* PRM exposure time p.371
poisson totalarts i.fem mar kid5 phd ment, nolog exposure(profage)

* PRM exposure time as rhs variable with a constraint p.371
constraint define 1 lnage=1
poisson totalarts i.fem mar kid5 phd ment lnage, nolog constraint(1)

* PRM with offset() p.372
poisson totalarts i.fem mar kid5 phd ment, nolog offset(lnage)

//  #8
//  nbrm - estimation p.372 / p.375

sysuse couart4, clear
nbreg art i.fem mar kid5 phd ment, nolog

* combining poisson and nbreg p.375/376
poisson art i.fem mar kid5 phd ment, nolog
estimates store PRM
nbreg art i.fem mar kid5 phd ment, nolog
estimates store NBRM
estimates table PRM NBRM, b(%9.3f) t label varwidth(32) ///
    drop(lnalpha:_cons) stats(alpha N)

//  #9
//  nbrm - interpretation using the rate mu p.377

*nbreg art i.fem mar kid5 phd ment, nolog
rlistcoef i.fem ment, help
rlistcoef i.fem ment, help percent

//  #10
//  nbrm - interpretation using predicted probabilities p.378

sysuse couart4, clear

* prvalue for prm and nbrm p.379
quietly poisson art i.fem mar kid5 phd ment
* rate and probabilities
qui mtable, atmeans
mtable, outcome(0(1)9) atmeans dec(4) right long

quietly nbreg art i.fem mar kid5 phd ment
* rate and probabilities
qui mtable, atmeans
mtable, outcome(0(1)9) atmeans dec(4) right long

* comparing probabilities for prm and nbreg
quietly {
poisson art i.fem mar kid5 phd ment
mtable, atmeans outcome(0(1)5) dec(4) noci roweq(PRM)
nbreg art i.fem mar kid5 phd ment
}
mtable, atmeans outcome(0(1)5) dec(4) noci roweq(NBRM) below

* pr(y=0) for prm vs nbrm p.380
quietly nbreg art i.fem mar kid5 phd ment, nolog
qui mgen, atmeans at(ment=(0(2.5)50)) stub(nb) outcome(0)
quietly poisson art i.fem mar kid5 phd ment
qui mgen, atmeans at(ment=(0(2.5)50)) stub(psn) outcome(0)
label var psnp0 "Pr(0) for PRM"
label var nbp0 "Pr(0) for NBRM"
graph twoway connected psnp0 nbp0 nbment, ///
    ylabel(0(.1).4) yline(.1 .2 .3) ///
    ytitle("Probability of a Zero Count")
graph export `exstub'08nbprm0.emf, replace

* pr(y=0) for prm vs nbrm with confidence intervals p.381
graph twoway ///
    (rarea psnlb0 psnub0 nbment, color(gs14)) ///
    (rarea nblb0 nbub0 nbment, color(gs14)) ///
    (connected psnp0 nbment, lpattern(dash) msize(zero)) ///
    (connected nbp0 nbment, lpattern(solid) msize(zero)), ///
    legend(on order(3 4)) ylabel(0(.1).4) yline(.1 .2 .3) ///
    ytitle("Probability of a Zero Count")
graph export `exstub'08nbprm0ci.emf, replace

//  #11
//  fitting zero truncated models p.381 (replacing ztp and ztnb)

/* options for predict in tpoisson and tnbreg

      n          number of events; the default
      ir         incidence rate
      cm         conditional mean, E(y | y > ll)
      pr(n)      probability Pr(y = n)
      pr(a,b)    probability Pr(a < y < b)
      cpr(n)     conditional probability Pr(y = n | y > ll)
      cpr(a,b)   conditional probability Pr(a < y < b | y > ll)
      xb         linear prediction
      stdp       standard error of the linear prediction
*/

sysuse couart4, clear
drop if art==0 // artificially truncated the data

//  fitting zero truncated models p.383

tpoisson art i.fem mar kid5 phd ment, nolog
est store tpoisson
tnbreg art i.fem mar kid5 phd ment, nolog
est store tnbreg

//  interpretation of parameters p.384

estimates table tpoisson tnbreg, stats(N chi2 aic bic) eform b(%9.3f) t(%6.2f)

//  interpretation using predicted probabilities and rates p.386

* unconditional mean and probabilities p.386
mtable, at(fem=1 mar=1 kid5=0) atmeans
mtable, at(fem=1 mar=1 kid5=0) atmeans outcome(0(1)7) width(5) dec(2) right

* conditional mean and probabilities p.386
mtable, at(fem=1 mar=1 kid5=0) atmeans predict(cm)
mtable, at(fem=1 mar=1 kid5=0) atmeans cpr outcome(1(1)6) width(6) right

* predict example: margins, at(fem=1 mar=1 kid5=0) atmeans predict(cpr(2))

//  computing predicted rates & probs in the estimation sample p.387

tnbreg art i.fem mar kid5 phd ment, nolog
predict tnbrate, ir
predict tnbcm, cm

forvalues i = 1(1)5 {
    predict tnbpr`i', pr(`i')
    predict tnbcpr`i', cpr(`i')
}
sum tnbpr* tnbcpr*

//  #12
//  nbrm hurdle model p.387

sysuse couart4, clear
gen iszero = (art==0)
label var iszero "Does scientists have 0 articles?"
label def iszero 1 NoArticles 0 HasArticles
label val iszero iszer

* binary logit for zero publications p.388
logit iszero i.fem mar-ment, nolog or
est store hlogit

* zero truncated negative binomial p.388
*FV error : tnbreg art i.fem-ment if art>0, nolog irr
tnbreg art i.fem mar-ment if art>0, nolog irr
est store htnbreg

* RM3 hilbe's hnblogit command
* FV: factor variables not allowed
hnblogit art fem mar-ment

* table of estimates from nbreg-logit hurdle model p.389
estout hlogit htnbreg, eq(1) style(fixed) ///
    cells(b(star fmt(%9.3f)) se(par fmt(%9.3f))) ///
    collabels("") mlabels("h_logit" "h_ztnb") ///
    eqlabels(,none) stats(N ll, fmt(%9.0f %9.3f)) ///
    varlabels(lnalpha:_cons _alpha) legend

* predict prob of 0 p.389
estimates restore hlogit
predict Hlgt_pry0, p
label var Hlgt_pry0 "blm: Pr(y=0)=Pr(iszero=1)"
summarize Hlgt_pry0

* prob of positive counts p.390
estimates restore htnbreg
predict tnbrate, ir
label var tnbrate "Incident rate from tnbreg at 0"
predict tnbcm, cm
label var tnbcm "Predicted conditional rate from tnbreg at 0"
forvalues i = 1(1)9 {
    predict tnbpr`i', pr(`i')
    predict tnbcpr`i', cpr(`i')
    label var tnbcpr`i' "Pr(y=`i'|y>0) from tnbreg at 0"
}
codebook tnb*, compact

* rate for hurle model p.390
gen Hmu = (1-Hlgt_pry0) * tnbcm  // Hztnb_Crate
label var Hmu "Hurdle: mu"
summarize Hmu

* probabilities from hurlde model p.390
gen Hpr0 = Hlgt_pry0
label var Hpr0 "Hurdle: pr(y=0)"
foreach k of numlist  1(1)9  {
    gen Hpr`k' = (1-Hlgt_pry0) * tnbcpr`k'
    label var Hpr`k' "Hurdle: pr(y=`k')"
}

* summary of predictions p.391
codebook Hpr*, compact

//  predictions for user specified values p.391

est restore hlogit
* pr0 p.391
mtable, at(fem=0 phd=1 ment=0) atmeans mlistat
matrix list _mtab_displayed
scalar hrdlp0 = _mtab_displayed[1,1]
display "Prob(art=0|X) = " hrdlp0

* RM3 DO: allow mtable to compute cpr()
* pr1,2,... p.392-393
est restore htnbreg
forvalues i = 1(1)9 {
    qui margins, atmeans at(fem=0 phd=1 ment=0) predict(cpr(`i'))
    scalar cp = el(r(b),1,1)
    scalar hrdlp = (1-hrdlp0) * cp
    display "Prob(art=`i'|X) = " hrdlp
}

//  #13
//  zip/zinb - estimating the zip and zinb models p.394

sysuse couart4, clear
zip art i.fem mar kid5 phd ment, inf(i.fem mar kid5 phd ment) nolog
estimates store ZIP
zinb art i.fem mar kid5 phd ment, inf(i.fem mar kid5 phd ment) nolog
estimates store ZINB
estimates table ZIP ZINB, b(%9.3f) t label varwidth(35)

//  #14
//  zip/zinb - interpretation of coefficients

zip art i.fem mar kid5 phd ment, inf(i.fem mar kid5 phd ment) nolog
rlistcoef, help
* zinb interpretation p.399
zinb art i.fem mar kid5 phd ment, inf(i.fem mar kid5 phd ment) nolog
estimates store zinb
rlistcoef, help

//  #15
//  zip/zinb - interpretation of predicted probabilities p.400

* discrete change without confidence intervals for zinb p.401

* rate p.401
mtable, at(fem=0 mar=1 kid5=3 phd=3 ment=10) atmeans ///
        at(fem=1 mar=1 kid5=3 phd=1 ment=0)

* DO: verify these 0 types
* probabilities of "degenerate zero" p.401
mtable, at(fem=0 mar=1 kid5=3 phd=3 ment=10) atmeans ///
        at(fem=1 mar=1 kid5=3 phd=1 ment=0) predict(pr) predname(All0)
* probability of 0 from any source
mtable, at(fem=0 mar=1 kid5=3 phd=3 ment=10) atmeans atoff ///
        at(fem=1 mar=1 kid5=3 phd=1 ment=0) outcome(0) ///
        width(6) right predname(Any0)

* probabilities p.401
*       note: probability of 0 from any source
mtable, at(fem=0 mar=1 kid5=3 phd=3 ment=10) atmeans ///
        at(fem=1 mar=1 kid5=3 phd=1 ment=0) outcome(0(1)9) width(6)

* tests of differences for two levels of at
forvalues i = 0(1)9 {
    estimates restore zinb
    margins, atmeans at(fem=0 mar=1 kid5=3 phd=3 ment=10) noatlegend ///
        at(fem=1 mar=1 kid5=3 phd=1 ment=0) post predict(pr(`i'))
    * do: show lincom command....
    mlincom , exp(2-1)
}

/* confidence intervals for zinb
* -- this will take about 1800 seconds to run in 2005; 321 in 2012
timer clear
timer on 1
set seed 14972132
set maxiter 50
prvalue, x(fem=0 mar=1 kid5=3 phd=3 ment=10) save boot dots
prvalue, x(fem=1 mar=1 kid5=3 phd=1 ment=0) diff boot dots
timer off 1
timer list
*/

* predictions for plotting p.304

zinb art i.fem mar kid5 phd ment, inf(i.fem mar kid5 phd ment) nolog

* probability of 0 whether always or sometimes
mgen , stub(zinbAny) predict(pr(0)) atmeans at(ment=(0(1)20)) noatlegend
    * margins note: could use outcome(0)
* probabilty of 0 from binary equation (always 0)
mgen , stub(zinbAlways) predict(pr) atmeans at(ment=(0(1)20)) noatlegend

gen zinbSome = zinbAnyp - zinbAlwaysp
label var zinbAnyp "0s from Both Equations"
label var zinbAlwaysp "0s from Binary Equation"
label var zinbSome  "0s from Count Equation"
label var zinbAnyment "Mentor's Publications"

* graphing types of 0 p.405
graph twoway connected zinbAlwaysp zinbSome zinbAnyp zinbAnyment, ///
    xlabel(0(5)20) ylabel(0(.1).7) ///
    ytitle(Probability of Zero) msymbol(Oh Sh O)
graph export `exstub'08zinbpr0.emf, replace

//  #16
//  comparing mean probabilities p.405

* estimate various models and save predictions p.406
sysuse couart4, clear
quietly poisson art i.fem mar kid5 phd ment, nolog
qui mgen, stub(prm) outcome(0(1)9) meanpred
label var prmpreq "Predicted: PRM"
label var prmobeq "Observed"
sum prmpreq
sum prmobeq

quietly nbreg art i.fem mar kid5 phd ment, nolog
qui mgen, stub(nbrm) outcome(0(1)9) meanpred
label var nbrmpreq "Predicted: NBRM"

quietly zip art i.fem mar kid5 phd ment, ///
    inf(i.fem mar kid5 phd ment) vuong nolog
qui mgen, stub(zip) outcome(0(1)9) meanpred
label var zippreq "Predicted: ZIP"

quietly zinb art i.fem mar kid5 phd ment, ///
    inf(i.fem mar kid5 phd ment) vuong nolog
qui mgen, stub(zinb) outcome(0(1)9) meanpred
label var zinbpreq "Predicted: ZINB"

* create deviations
gen obs = prmobeq
gen dprm = obs - prmpreq
label var dprm "PRM"
gen dnbrm = obs - nbrmpreq
label var dnbrm "NBRM"
gen dzip = obs - zippreq
label var dzip "ZIP"
gen dzinb = obs - zinbpreq
label var dzinb "ZINB"

* plot deviations p.407
graph twoway connected dprm dnbrm dzip dzinb prmval, ///
    ytitle(Observed-Predicted) ylabel(-.10(.05).10) ///
    xlabel(0(1)9) msymbol(Oh Sh O S)
graph export `exstub'08compare.emf, replace

//  #17
//  tests to compare count models p.407

* LR test for zip and zinb p.407-408
sysuse couart4, clear
quietly zip art i.fem mar kid5 phd ment, inf(i.fem mar kid5 phd ment) vuong nolog
scalar llzip = e(ll)
quietly zinb art i.fem mar kid5 phd ment, inf(i.fem mar kid5 phd ment) vuong nolog
scalar llzinb = e(ll)
scalar lr = -2*(llzip-llzinb)
scalar pvalue = chiprob(1,lr)/2
scalar lnalpha = -.9763565
if (lnalpha < -20) {
    scalar pvalue= 1
}
di as text "Likelihood-ratio test comparing ZIP to ZINB: " as res %8.3f ///
    lr as text " Prob>=" as res %5.3f pvalue

* vuong test of non-nested models p.408
zip art i.fem mar kid5 phd ment, inf(i.fem mar kid5 phd ment) vuong nolog
rlistcoef, help
zinb art i.fem mar kid5 phd ment, inf(i.fem mar kid5 phd ment) vuong nolog
rlistcoef, help

//  #18
//  countfit to compare count models p.410

sysuse couart4, clear
rcountfit art i.fem mar kid5 phd ment, ///
    inf(ment i.fem) graphexport(08countfigeg.emf, replace)
graph export `exstub'08countfigeg.emf, replace

log close
exit
