*! version 3.0.5 2012-07-30 shiia df

capture program drop rmlogtest
program define rmlogtest, rclass

    version 11.2
    tempname numrhs sample numcats omit chisq df pval v rowres matsh touse
    tempname matiia matsuest matlr matwald matcomb matlrc nxtrow testn newbase
    tempname Vals b0a b0b b0ab b1b
    tempvar tmp count samp tempy tempvar lnL denom 

    syntax [varlist(default=none)][, Detail Iia Hausman SMhsiao SUest ///
        Lr Wald Combine LRComb Set(string) All Base Debug(integer 0)]

    if "`e(cmd)'" != "mlogit" {
        di _newline in y "mlogtest" in r " only works after " in y "mlogit."
        exit
    }
    qui estimates store _Orig 

    // formatting setup

    local n "(N=`e(N)')"
    local smcl "in smcl "
    local dash "{c -}"
    local vlin "{c |}"
    local plus "{c +}"
    local topT "{c TT}"
    local botT "{c BT}"

    //  option indicators

    local dolr = "no"
        if ("`lr'"!="") local dolr = "yes"
    local doiia = "no"
        if ("`hausman'"!="" | "`iia'"!="") local doiia = "yes"
    local dosmhsiao = "no"
        if ("`smhsiao'"!="" | "`iia'"!="") local dosmhsiao = "yes"
    local dosuest = "no"
        if ("`suest'"!="" | "`iia'"!="") local dosuest = "yes"
    local dowald = "no"
    if ("`wald'"!="") local dowald = "yes"
    local docombine = "no"
        if ("`combine'"!="") local docombine = "yes"
    local dolrcomb = "no"
        if ("`lrcomb'"!="") local dolrcomb = "yes"
    if "`all'"!="" {
        local dolr = "yes"
        local doiia = "yes"
        local dosmhsiao = "yes"
        local dosuest = "yes"
        local dowald = "yes"
        local docombine = "yes"
        local dolrcomb = "yes"
    }
    
*-> get weight info from last mlogit

    local wtis ""
    if ("`e(wtype)'"!="") local wtis "[`e(wtype)'`e(wexp)']"

*-> check that estimation sample matches n from regression

    qui gen `sample' = e(sample)
    if "`e(wtype)'"=="" | "`e(wtype)'"=="aweight" | "`e(wtype)'"=="pweight" {
        qui count if `sample' == 1
        scalar `testn' = r(N)
    }
    if "`e(wtype)'"=="fweight" | "`e(wtype)'"=="iweight" {
        local wtexp = substr("`e(wexp)'", 3, .)
        gen `tmp' = (`wtexp')*`sample'
        qui su `tmp', meanonly
        scalar `testn' = round(r(sum),1)
    }
    if e(N) != `testn' {
        di  _newline in r "Data has been altered since " ///
            in y "mlogit" in r " was estimated."
        exit
    }

*-> get information about the mlogit

    local depvar = "`e(depvar)'"
    local printdepvar = abbrev("`printdepvar'", 8)

    rm3_perhs, noomit // 301
    local rhsnam "`r(rhsnms)'" // excludes omitted variables
    scalar `numrhs' = `r(nrhs)'

    rm3_pecats
    local catnms8 "`r(catnms8)'"
    local catvals "`r(catvals)'"
    local catnms "`r(catnms)'"
    scalar `numcats' = r(numcats)
    local basecat = e(baseout)

    local refnm "`r(refnm)'"
    local check : word count `catvals'
    if `numcats' != real("`check'") {
        di  _newline in r "Problem determining number of outcome categories."
        exit        
    }

*-> parse sets if set option on

    local numset = 0
    if "`set'" != "" {
        tokenize "`set'", parse("\")
        local count = 1
        while "``count''"!="" {
            if "``count''"=="\" {
                macro shift
            }
            else {
                local set`count' "``count''"
                capture  unab set`count': ///
                    `set`count'', min(2) name(defining sets)
                local setl`count': word count `set`count''
                * check if all variables in set are rhs variables
                local count2 = 1
                while `count2' <= `setl`count'' {
                    local count3 = 1
                    local inrhs "no"
                    local setvar : word `count2' of `set`count''
                    while `count3' <= `numrhs' {
                        local ivar : word `count3' of `rhsnam'
                        if ("`setvar'"=="`ivar'") {
                            local inrhs "yes"
                        }
                        local count3 = `count3' + 1
                    }
                    if "`inrhs'" == "no" {
                        di _newline in r "variable " in y "`setvar'" in r ///
                            " specified in set() but is not in model."
                        exit
                    }
                    local count2 = `count2' + 1
                } // while count2 <= `setl`count'
                return local set_`count' "`set`count''"
                local count = `count' + 1
            } // else 
        } // while "``count''"!=""
        local numset = `count' - 1
    } // if "`set'" != "" 

*-> LR test of independent variables

*! version 11 makes this section of code 11 times slower than version 10!
    if "`dolr'"=="yes" & `numrhs' == 0 {
        di _n in r "LR test cannot be computed for an intercept-only model."
    }
    else if "`dolr'"=="yes" {
        di _newline in g ///
        "Likelihood-ratio tests for independent variables `n'"
        di _newline in g ///
        "  Ho: All coefficients associated with given variable(s) are 0."
        di `smcl' _newline in g  %12s "`printdepvar'" _col(14) "`vlin'" ///
            _col(22) "chi2" _col(29) "df" _col(34) "P>chi2"
        di `smcl' in g _dup(13) "`dash'" "`plus'" _dup(25) "`dash'"

        * allow tests of specified # of variables
        if "`varlist'"!="" {
            local nvlist : word count `varlist'
            local ntests = `nvlist' // use this as the # of rhsvars
            local testnms "`varlist'"
        }
        else {
            local testnms "`rhsnam'"
            local ntests = `numrhs'
        }
        tokenize "`rhsnam'" // tokenize list, rather than rhs vars

        * loop over all rhs + specified sets
        local count = 1
        while `count' <= `ntests'+`numset' {

            local testvar : word `count' of `testnms'
            * for individual independent variables
            if `count' <= `ntests' {
                local var`count' : word `count' of `testnms'
                * create varlist with all vars but count variable
                local count2 = 1
                local lrrhs = ""
                while `count2' <= `numrhs' { // loop through all variables
                    if "`testvar'"!="``count2''" {
                        local lrrhs "`lrrhs' ``count2''"
                    }
                    local count2 = `count2' + 1
                }
            } // if `count' <= `ntests'

            * for sets of independent variables
            if `count' > `ntests' {
                local thisset = `count'-`ntests'
                * for matrix & output rowname
                local var`count' "set_`thisset'"
                local countto : word count `set`thisset''
                local count2 = 1
                local lrrhs = ""
                while `count2' <= `ntests' {
                    local count3 = 1
                    local inset "no"
                    while `count3' <= `countto' {
                        local setvar : word `count3' of `set`thisset''
                        if ("``count2''"=="`setvar'") local inset "yes"
                        local count3 = `count3' + 1
                    }
                    if ("`inset'"=="no") local lrrhs "`lrrhs' ``count2''"
                    local count2 = `count2' + 1
                }
            } // if `count' > `ntests'

            qui mlogit `depvar' `lrrhs' `wtis' if `sample' == 1, b(`basecat')
            qui lrtest . _Orig
            scalar `chisq' = r(chi2)
            scalar `df' = r(df)
            scalar `pval' = r(p)
            if (`pval'==.) { 
                scalar `pval' = -9999
            }
            mat `nxtrow' = `chisq', `df', `pval'
            mat rownames `nxtrow' = "`var`count''"
            mat `matlr' = nullmat(`matlr') \ `nxtrow'
            local count = `count' + 1
        } // while `count' <= `ntests'
        mat colnames `matlr' = chi2 df p
        * cycle through output matrix and print
        local countto = rowsof(`matlr')
        local count = 1
        while `count' <= `countto' {
            scalar `chisq' = `matlr'[`count', 1]
            scalar `df' = `matlr'[`count', 2]
            scalar `pval' = `matlr'[`count', 3]
            if (`pval'==-9999) { 
                scalar `pval' = . 
            }

            * tests of individual variables
            if `count' <= `ntests' {
                local printnm "`var`count''"
                local printnm = abbrev("`printnm'", 12)
                di `smcl' in y %12s "`printnm'" _col(14) in g "`vlin'" ///
                    _col(17) %9.3f in y `chisq' _col(26) %5.0f `df' ///
                    _col(35) %4.3f `pval'
            }

            * tests of sets
            if `count' > `ntests' {
                local thisset = `count' - `ntests'
                di `smcl' in g _dup(13) "`dash'" "`plus'" _dup(25) "`dash'"
                local printnm "`var`count''"
                local printnm = abbrev("`printnm'", 12)
                di `smcl' in g %12s "`printnm':" _col(14) in g "`vlin'" ///
                    _col(17) %9.3f in y `chisq' _col(26) %5.0f `df' ///
                    _col(35) %4.3f `pval'
                local count2 = 1
                local countt2 : word count `set`thisset''
                while `count2' <= `countt2' {
                    local setvar : word `count2' of `set`thisset''
                    local printnm "`setvar'"
                    local printnm = abbrev("`printnm'", 12)
                    di `smcl' %12s in y "`printnm'" _col(14) in g "`vlin'"
                    local count2 = `count2' + 1
                }
            }
            local count = `count' + 1
        } // while `count' <= `countto'
        di `smcl' in g _dup(13) "`dash'" "`botT'" _dup(25) "`dash'"
        qui estimates restore _Orig
        return matrix lrtest `matlr'

    } // if "`dolr'"=="yes"

*-> WALD test of independent variables

    if "`varlist'"!="" {
        local nvlist : word count `varlist'
        local testnms "`varlist'"
        local ntests = `nvlist' // use this as the # of rhsvars
    }
    else {
        local ntests = `numrhs'
        local testnms "`rhsnam'"
    }

    if "`dowald'"=="yes" & `ntests' == 0 {
        di _n in r "Wald test cannot be computed on intercept-only model."
        exit        
    }
    else if "`dowald'"=="yes" {
        tokenize "`testnms'"
        di _newline in g "Wald tests for independent variables `n'"
        di _newline in g ///
            "  Ho: All coefficients associated with given variable(s) are 0."
        di `smcl' _newline in g  %12s "`printdepvar'" _col(14) "`vlin'" ///
            _col(22) "chi2" _col(29) "df" _col(34) "P>chi2"
        di `smcl' in g _dup(13) "`dash'" "`plus'" _dup(25) "`dash'"

        * loop over all rhs + sets
        local count = 1
        while `count' <= `ntests'+`numset' {
            if `count' <= `ntests' {
                local var`count' : word `count' of `testnms'
                qui test `var`count''
            }
            * tests of sets
            if `count' > `ntests' {
                local thisset = `count'-`ntests'
                *get set name/number for matrix&output
                local var`count' "set_`thisset'"
                qui test `set`thisset''
            }
            scalar `chisq' = r(chi2)
            scalar `df' = r(df)
            scalar `pval' = r(p)
            if (`pval'==.) { 
                scalar `pval' = -9999 
            }
            mat `nxtrow' = `chisq', `df', `pval'
            mat rownames `nxtrow' = "`var`count''"
            mat `matwald' = nullmat(`matwald') \ `nxtrow'
            local count = `count' + 1
        } // while `count' <= `ntests'

        mat colnames `matwald' = chi2 df p
        local countto = rowsof(`matwald')
        local count = 1
        while `count' <= `countto' {
            scalar `chisq' = `matwald'[`count', 1]
            scalar `df' = `matwald'[`count', 2]
            scalar `pval' = `matwald'[`count', 3]
            if (`pval'==-9999) { 
                scalar `pval' = . 
            }
            if `count' <= `ntests' {
                local printnm "`var`count''"
                local printnm = abbrev("`printnm'", 12)
                di `smcl' in y %12s "`printnm'" _col(14) in g "`vlin'" ///
                    _col(17) %9.3f in y `chisq' _col(26) %5.0f `df' ///
                    _col(35) %4.3f `pval'
            }
            if `count' > `ntests' {
                local thisset = `count' - `ntests'
                di `smcl' in g _dup(13) "`dash'" "`plus'" _dup(25) "`dash'"
                local printnm "`var`count''"
                local printnm = abbrev("`printnm'", 12)
                di `smcl' in g %12s "`printnm':" _col(14) in g "`vlin'" ///
                    _col(17) %9.3f in y `chisq' _col(26) %5.0f `df' ///
                    _col(35) %4.3f `pval'
                local count2 = 1
                local countt2 : word count `set`thisset''

                while `count2' <= `countt2' {
                    local setvar : word `count2' of `set`thisset''
                    local printnm "`setvar'"
                    local printnm = abbrev("`printnm'", 12)
                    di `smcl' %12s in y "`printnm'" _col(14) in g "`vlin'"
                    local count2 = `count2' + 1
                }
            }
            local count = `count' + 1
        } // while `count' <= `countto'
        di `smcl' in g _dup(13) "`dash'" "`botT'" _dup(25) "`dash'"
        return matrix wald `matwald'
    } // if "`dowald'"=="yes"

*-> HAUSMAN IIA Test

    if "`doiia'"=="yes" & `numcats' == 2 {
        di _n in r ///
            "Hausman IIA test requires at least 3 dependent categories."
        exit            
    }
    else if "`doiia'"=="yes" {
        di _newline in g "Hausman tests of IIA assumption `n'"
        di _newline in g ///
        "  Ho: Odds(Outcome-J vs Outcome-K) are independent of " ///
        "other alternatives."

        * cycle through all alternatives
        tokenize "`catvals'"
        local count = 1
        while real("`count'") <= `numcats' {

            local lab`count' : word `count' of `catnms'
            local slab`count' : word `count' of `catnms8'
            scalar `omit' = real("``count''")

            if "`detail'" == "detail" {
                if "`lab`count''"!="``count''" {
                    di _newline in g ///
                        "Hausman test when omitted alternative is " ///
                        in y `omit' in g " (" in y "`lab`count''" in g ")"
                }
                else {
                    di in g _newline ///
                       "Hausman test when omitted alternative is " in y `omit'
                }
            } // if "`detail'" == "detail" 

            if `omit'==real("`basecat'") & "`base'" != "" {
                * IIA for basecat requires estimating mlogit with
                * new basecategory. Make new basecat the largest category
                * of the dependent variable that is not the original basecat
                local maxcnt = 0
                local count2 = 1
                while `count2' <= `numcats' {
                    if real("``count2''") != `omit' {
                        qui count if `depvar'==``count2'' & `sample'==1
                        if r(N)>`maxcnt' {
                            local newbase = ``count2''
                            local maxcnt = r(N)
                            local tmplab : word `count2' of `catnms'
                        }
                    }
                    local count2 = `count2' + 1
                } // while count2 <= `numcats'

                qui mlogit `depvar' `rhsnam' `wtis' ///
                    if `sample' == 1, b(`newbase')
                qui estimates store _OrigNewBase
                qui mlogit `depvar' `rhsnam' `wtis' ///
                    if `sample'==1 & `depvar'!=`omit', b(`newbase')

                if "`detail'"=="detail" {
                    if "`lab`count''"!="``count''" {
                        di in g "(Using category " in g "`newbase' " ///
                          in g "(" in g "`tmplab'" in g ") as comparison group)"
                    }
                    else {
                        di in g "(Using category " in g "`newbase' " ///
                          in g "as comparison group)"
                    }
                    hausman . _OrigNewBase, alleq constant
                }
                else {
                    qui hausman . _OrigNewBase, alleq constant
                }
                scalar `chisq' = r(chi2)
                scalar `df' = r(df)
                scalar `pval' = r(p)
                if (`pval'==. | `chisq'<0) { 
                    scalar `pval' = -9999 
                }

                qui estimates restore _Orig

                mat `nxtrow' = `omit', `chisq', `df', `pval'
                mat rownames `nxtrow' = "`slab`count''"
                mat `matiia' = nullmat(`matiia') \ `nxtrow'

            } // if `omit'==real("`basecat'")

            else if `omit'!=real("`basecat'") {
                qui mlogit `depvar' `rhsnam' `wtis' ///
                    if `sample' == 1 & `depvar' != `omit', b(`basecat')
                if "`detail'"=="detail" {
                    hausman . _Orig, alleq constant
                }
                else {
                qui hausman . _Orig, alleq constant
                }
                scalar `chisq' = r(chi2)
                scalar `df' = r(df)
                scalar `pval' = r(p)
                if `pval'==. | `chisq'<0 { 
                    scalar `pval' = -9999 
                }
                mat `nxtrow' = `omit', `chisq', `df', `pval'
                mat rownames `nxtrow' = "`slab`count''"
                mat `matiia' = nullmat(`matiia') \ `nxtrow'
            } // else if `omit'!=real("`basecat'")
            local count = `count' + 1
        } // while real("`count'") <= `numcats'

        mat colnames `matiia' = omitted chi2 df p

        if "`detail'"=="detail" {
            di in g _newline "Summary of results"
        }

        * only print explainer if negative chi2 observed
        local anyneg = "no"
        local countto = rowsof(`matiia')
        local count = 1

        di _newline `smcl' in g " Omitted" _col(10) "`vlin'" ///
          _col(17) "chi2" _col(24) "df" _col(29) "P>chi2" _col(38) "evidence"
        di `smcl' in g _dup(9) "`dash'" "`plus'" _dup(36) "`dash'"

        *cycle by row and print
        while `count' <= `countto' {
            scalar `chisq' = `matiia'[`count', 2]
            scalar `df' = `matiia'[`count', 3]
            scalar `pval' = `matiia'[`count', 4]
            if `pval' == -9999 {
                local anyneg "yes"
                local implies "---"
                local printnm "`slab`count''"
                local printnm = abbrev("`printnm'", 8)
                di `smcl' in y %8s "`printnm'" _col(10) in g "`vlin'" ///
                  _col(13) %8.3f in y `chisq' _col(21) %5.0f `df' ///
                  _col(30) " ---" _col(38) %-10s "`implies'"
            }
            else {
                local implies  "for Ho"
                if `pval'<.05 {
                    local implies "against Ho"
                }
                local printnm "`slab`count''"
                local printnm = abbrev("`printnm'", 12)
                di `smcl' %8s in y "`printnm'" _col(10) in g "`vlin'" ///
                    _col(13) %8.3f in y `chisq' _col(21) %5.0f `df' ///
                    _col(30) %4.3f `pval' _col(38) %-10s "`implies'"
            }
            local count = `count' + 1
        } // while `count' <= `countto'
        di `smcl' in g _dup(9) "`dash'" "`botT'" _dup(36) "`dash'"

        if "`anyneg'" == "yes" {
            di in g "Note: If chi2<0, the estimated model does not "
            di in g "meet asymptotic assumptions."
        }

        qui estimates restore _Orig
        return matrix hausman `matiia'

    } // if "`doiia'"=="no"

*-> suest-based HAUSMAN IIA Test

    * see if cluster used; if so incorporate in suest
    local clustvar = e(clustvar)

    if "`dosuest'"=="yes" & `numcats' == 2 {
        di _n in r ///
          "suest-based Hausman IIA test requires at least 3 outcome categories."
    }
    else if "`clustvar'" != "." & "`clustvar'" != "" {
        di _n as err ///
"suest-based Hausman IIA test must be estimated differently when cluster()"
        di as err "is specified; see [R] suest"
    }
    else if "`dosuest'"=="yes" {

        di _newline in g ///
          "suest-based Hausman tests of IIA assumption `n'"
        di _newline in g ///
          "  Ho: Odds(Outcome-J vs Outcome-K) are independent of "  ///
          "other alternatives."
        * cycle through all alternatives
        tokenize "`catvals'"
        local count = 1
        while real("`count'") <= `numcats' {

            local lab`count' : word `count' of `catnms'
            local slab`count' : word `count' of `catnms8'
            scalar `omit' = real("``count''")
            if `omit'==real("`basecat'") & "`base'" != "" {

                * IIA for basecategory requires estimating mlogit with
                * new basecategory. Make new basecat the largest category
                * of the dependent variable that is not the original basecat
                local maxcnt = 0
                local count2 = 1
                while `count2' <= `numcats' {
                    if real("``count2''") != `omit' {
                        qui count if `depvar'==``count2'' & `sample'==1
                        if r(N)>`maxcnt' {
                            local newbase = ``count2''
                            local maxcnt = r(N)
                            local tmplab : word `count2' of `catnms'
                        }
                    }
                    local count2 = `count2' + 1
                } // while count2 <= `numcats'

                * estimates of old logit are held in _Orig
                qui mlogit `depvar' `rhsnam' `wtis' ///
                    if `sample' == 1, b(`newbase')
                qui estimates store _OrigNewBase
                qui mlogit `depvar' `rhsnam' `wtis' ///
                    if `sample'==1 & `depvar'!=`omit', b(`newbase')
                qui estimates store _Aux
                local eqnames = e(eqnames)
                qui estimates store _Aux
                qui suest _Aux _OrigNewBase
                qui estimates store _suest

                foreach name in `eqnames' {
                    qui test [_OrigNewBase_`name' = _Aux_`name'], ///
                        cons accum
                }
                if "`detail'"=="detail" {
                    suest
                    test
                }

                scalar `chisq' = r(chi2)
                scalar `df' = r(df)
                scalar `pval' = r(p)
                if (`pval'==. | `chisq'<0) { 
                    scalar `pval' = -9999 
                }

                qui estimates restore _Orig
                mat `nxtrow' = `omit', `chisq', `df', `pval'
                mat rownames `nxtrow' = "`slab`count''"
                mat `matsuest' = nullmat(`matsuest') \ `nxtrow'

            } // if `omit'==real("`basecat'")

            else if `omit'!=real("`basecat'") {
                qui mlogit `depvar' `rhsnam' `wtis' ///
                    if `sample' == 1 & `depvar' != `omit', b(`basecat')
                local eqnames = e(eqnames)
                qui estimates store _Aux
                qui suest _Aux _Orig
                qui estimates store _suest
                foreach name in `eqnames' {
                   * 302 common added for use with factor variables
                   qui test [_Orig_`name' = _Aux_`name'], cons accum common
                }
                if "`detail'"=="detail" {
                    suest
                    test
                }

                scalar `chisq' = r(chi2)
                scalar `df' = r(df)
                scalar `pval' = r(p)
                if (`pval'==. | `chisq'<0) { 
                    scalar `pval' = -9999 
                }
                qui estimates restore _Orig
                mat `nxtrow' = `omit', `chisq', `df', `pval'
                mat rownames `nxtrow' = "`slab`count''"
                mat `matsuest' = nullmat(`matsuest') \ `nxtrow'

            } // else if `omit'!=real("`basecat'")
            local count = `count' + 1

        } // while real("`count'") <= `numcats'

        mat colnames `matsuest' = omitted chi2 df p
        local anyneg = "no"
        local countto = rowsof(`matsuest')
        local count = 1

        di _newline `smcl' in g " Omitted" _col(10) "`vlin'" ///
          _col(17) "chi2" _col(24) "df" _col(29) "P>chi2" _col(38) "evidence"
        di `smcl' in g _dup(9) "`dash'" "`plus'" _dup(36) "`dash'"

        *cycle through rows and print
        while `count' <= `countto' {
            scalar `chisq' = `matsuest'[`count', 2]
            scalar `df' = `matsuest'[`count', 3]
            scalar `pval' = `matsuest'[`count', 4]
            if `pval' == -9999 {
                local anyneg "yes"
                local implies "---"
                local printnm "`slab`count''"
                local printnm = abbrev("`printnm'", 8)
                di `smcl' in y %8s "`printnm'" _col(10) in g "`vlin'" ///
                  _col(13) %8.3f in y `chisq' _col(21) %5.0f `df' ///
                  _col(30) " ---" _col(38) %-10s "`implies'"
            }
            else {
                local implies  "for Ho"
                if (`pval'<.05) {
                    local implies "against Ho"
                }
                local printnm "`slab`count''"
                local printnm = abbrev("`printnm'", 12)
                di `smcl' %8s in y "`printnm'" _col(10) in g "`vlin'" ///
                  _col(13) %8.3f in y `chisq' _col(21) %5.0f `df' ///
                  _col(30) %4.3f `pval' _col(38) %-10s "`implies'"
            }
            local count = `count' + 1
        } // while `count' <= `countto'
        di `smcl' in g _dup(9) "`dash'" "`botT'" _dup(36) "`dash'"

        qui estimates restore _Orig
        return matrix suest `matsuest'

    }  // if "`doiia'"=="no"

*-> Small-Hsiao test of iia

    * NOTE: THE CODE FOR COMPUTING THE SMALL-HSIAO TEST IS ADAPTED
    * FROM CODE BY NICK WINTER IN HIS -SMHSIAO- COMMAND

    if "`dosmhsiao'"=="yes" & `numcats' == 2 {
        di _n in r ///
        "Small-Hsiao IIA test requires at least 3 dependent categories."
    }

    else if "`dosmhsiao'"=="yes" {
        di _newline in g "Small-Hsiao tests of IIA assumption `n'"
        di _newline in g ///
          "  Ho: Odds(Outcome-J vs Outcome-K) are independent of other alternatives."

        qui gen `touse' = e(sample)
        qui gen `samp'=round(uniform(),1)+1 if `touse'
        local y `e(depvar)'
        qui gen `tempy' = `depvar'
        local varlist "`tempy' `rhsnam'"

        qui ta `tempy' if `touse' & `samp'==1
        local cat1 `r(r)'
        qui ta `tempy' if `touse' & `samp'==2
        local cat2 `r(r)'
        if `cat1'!=`numcats' | `cat2'!=`numcats' {
            di in r ///
              "Random draw yielded empty cells for some categories of `y' in"
            di in r ///
              "one of the half-samples. Could not estimate Small-Hsiao test."
            error 148
        }

        local count = 1
        local countto = `numcats' - 1
        if ("`base'"!="") {
            local countto = `numcats'
        }

        while `count' <= `countto' {

            local bcat "b(`basecat')"
            if `count'==`numcats' {
                local bcat ""
            }
            local elim : word `count' of `catvals'
            local ielim `elim'
            qui tab `tempy' if `touse', matrow(`Vals')
            local nEvals = `numcats'-1
            local Yvals = ""
            local EYvals = ""
            local EYeqs = ""
            local i = 1
            while `i' <= `numcats' {
                local Yval`i' = `Vals'[`i',1]
                local Yvals "`Yvals' `Yval`i''"
                    if `Yval`i'' != `elim' {
                        local EYvals "`EYvals' `Yval`i''"
                        local EYeqs "`EYeqs' `i'"
                        local Ylab`i' `Yval`i''
                    }
                local i = `i' + 1
            }

            tempvar lnL denom
            tempname b0a b0b b0ab b1b

            qui mlogit `varlist' ///
                if (`touse' & `samp'==2 & `tempy'!=`elim') `wtis',  `bcat'
            mat `v' = e(V)
            * local dof = rowsof(`v')
            _ms_omit_info `v' // count o. columns in v
            local dof = rowsof(`v') - r(k_omit) // v305
           local lnL_1 = e(ll)
            if `count'==`numcats' {
                local tmpbcat "`e(basecat)'"
                local bcat "b(`tmpbcat')"
            }
            * estimate for each half sample
            qui mlogit `varlist' if `touse' & `samp'==1 `wtis', `bcat'
            mat `b0a' = e(b)
            qui mlogit `varlist' if `touse' & `samp'==2 `wtis', `bcat'
            mat `b0b' = e(b)
            * Zhang & Hoffman eq. 9
            mat `b0ab' = (0.70710678)*(`b0a') + (0.29289322)*(`b0b')

            * get LnL for amalgamated coefficients
            * get XBs & assemble denominator
            qui gen double `denom' = 0 if `touse'
            local i 1
            * cycle through values (w/o eliminated one)
            while `i' <= (`numcats'-1) {
                local cury : word `i' of `EYvals'
                local cureq : word `i' of `EYeqs'
                tempvar xb`cureq'
                if `cury' != `basecat' & "`cury'" != "`tmpbcat'" {
                    matrix score double `xb`cureq'' = `b0ab' ///
                        if `touse', eq(`Ylab`cureq'')
                }
                else {
                    qui gen double `xb`cureq''=0  // because (exp(0)=1)
                }
                qui replace `denom'=`denom' + exp(`xb`cureq'') if `touse'
                local i=`i'+1
            }

            * Log likelihood using amalgamated coeff
            qui gen double `lnL' = . if `touse'
            local i 1
            while `i'<=`nEvals' {
                local cureq : word `i' of `EYeqs'
                local cury : word `i' of `EYvals'
                qui replace `lnL' = ln(exp(`xb`cureq'')/(`denom')) ///
                    if `tempy'==`cury' & `touse'
                local i=`i'+1
            }

        * GET LnL, only for observations in the 2d sample
        * and without eliminated observations:
            sum `lnL' if `touse' & `samp'==2 & `tempy'!=`elim', meanonly
            local lnL_0 = r(sum)
            local SH = -2 * (`lnL_0' - `lnL_1')
            local p = chiprob(`dof',`SH')
            mat `rowres' = `elim' , `lnL_0' , `lnL_1' , `SH' , `dof' , `p'
            mat rownames `rowres' = "test `count'"
            mat colnames `rowres' = elim_cat lnL_0 lnL_1 chi2 df p
            mat `matsh' = nullmat(`matsh') \ `rowres'
            local count = `count' + 1
        }

        di _newline `smcl' in g " Omitted" _col(10) "`vlin'" ///
            _col(13) "lnL(full)" _col(24) "lnL(omit)" _col(37) "chi2" ///
            _col(44) "df" _col(49) "P>chi2" _col(58) "evidence"
        di `smcl' in g _dup(9) "`dash'" "`plus'" _dup(57) "`dash'"
        local count = 1
        local countto = rowsof(`matsh')
        while `count' <= `countto' {
            local elim = `matsh'[`count', 1]
            local elimnm8 : word `count' of `catnms8'
            local lnL_0 = `matsh'[`count', 2]
            local lnL_1 = `matsh'[`count', 3]
            local SH = `matsh'[`count', 4]
            scalar `df' = `matsh'[`count', 5]
            scalar `pval' = `matsh'[`count', 6]
            local count = `count' + 1
            local implies  "for Ho"
            if (`pval'<.05) {
                local implies "against Ho"
            }
            di `smcl' in y %8s "`elimnm8'" _col(10) in g "`vlin'" ///
                _col(13) %9.3f in y `lnL_0' _col(24) %9.3f `lnL_1' ///
                _col(33) %8.3f `SH' _col(41) %5.0f `df' ///
                _col(50) %4.3f `pval' _col(58) %-10s "`implies'"
        }
        di `smcl' in g _dup(9) "`dash'" "`botT'" _dup(57) "`dash'"

        qui mat list `matsh'
        qui estimates restore _Orig
        return matrix smhsiao `matsh'

    } // if "`dosmhsiao'"=="no"

*-> Wald tests for combining categories

    if "`docombine'"=="yes" & `numcats' == 2 {
        di in r "Test requires at least 3 dependent categories."
    }
    else if "`docombine'"=="yes" {

        di _newline in g ///
          "Wald tests for combining alternatives `n'"
        di _newline in g ///
          "  Ho: All coefficients except intercepts associated " ///
          "with a given pair"
        di in g ///
          "      of alternatives are 0 (i.e., alternatives can be combined)."
        di `smcl' _newline in g %17s "Alternatives tested" _col(20) ///
          "`vlin'" _col(26) "chi2" _col(33) "df" _col(38) "P>chi2"
        di `smcl' in g _dup(19) "`dash'" "`plus'" _dup(24) "`dash'"

        tokenize "`catvals'"
        qui estimates restore _Orig

        * cycle through all pairs of outcomes
        local count1 = 1
        while `count1' <= (`numcats'-1) {
            local count2 = `count1' + 1
            while `count2' <= `numcats' {
                    if ("``count1''"=="`basecat'") { 
                        quietly test [``count2''] 
                    }
                    else if "``count2''"=="`basecat'" { 
                        quietly test [``count1'']
                    }
                    else { 
                        quietly test [``count2''=``count1''] 
                    }
                    scalar `chisq' = r(chi2)
                    scalar `df' = r(df)
                    scalar `pval' = r(p)
                    local numrow = `numrow' + 1
                    local s1`numrow' : word `count1' of `catnms8'
                    local s2`numrow' : word `count2' of `catnms8'
                    if (`pval'==.) { 
                        scalar `pval' = -9999 
                    }
                    mat `nxtrow' = ``count1'', ``count2'', ///
                        `chisq', `df', `pval'
                    mat roweq `nxtrow' = "`s1`numrow''"
                    mat rownames `nxtrow' = "`s2`numrow''"
                    mat `matcomb' = nullmat(`matcomb') \ `nxtrow'
                local count2 = `count2' + 1
            }
            local count1 = `count1' + 1
        } // while `count1' <= `numcats'

        mat colnames `matcomb' = cat1 cat2 chi2 df p
        local countto = rowsof(`matcomb')
        local count = 1
        while `count' <= `countto' {
            scalar `chisq' = `matcomb'[`count', 3]
            scalar `df' = `matcomb'[`count', 4]
            scalar `pval' = `matcomb'[`count', 5]
            if (`pval'==-9999) { 
                scalar `pval' = . 
            }
            di `smcl' in y %8s "`s1`count''" _col(9) "-" %8s "`s2`count''" ///
              _col(20) in g "`vlin'" _col(21) %9.3f in y `chisq' ///
              _col(30) %5.0f `df' _col(39) %4.3f `pval'
            local count = `count' + 1
        }
        di `smcl' in g _dup(19) "`dash'" "`botT'" _dup(24) "`dash'"
        qui estimates restore _Orig
        return matrix combine `matcomb'
    } // if "`docombine'"=="yes"

*-> LR tests for combining categories

    if "`dolrcomb'"=="yes" & `numcats' == 2 {
        di in r "Test requires at least 3 dependent categories."
        exit
    }
    else if "`dolrcomb'"=="yes" {

        di _newline in g ///
          "LR tests for combining alternatives `n'"
        di _newline in g ///
          "  Ho: All coefficients except intercepts associated" ///
          " with a given pair"
        di in g ///
          "     of alternatives are 0 (i.e., alternatives can be collapsed)."
        di `smcl' _newline in g %17s "Alternatives tested" _col(20) ///
          "`vlin'" _col(26) "chi2" _col(33) "df" _col(38) "P>chi2"
        di `smcl' in g _dup(19) "`dash'" "`plus'" _dup(24) "`dash'"
        estimates store _LRbase        

if `debug'>2 {
    di _new "#11 mlogit of base"
    mlogit
}
        tokenize "`catvals'"

        * cycle through all pairs of outcomes
        local Nconstraint = 1
        while `Nconstraint' <= (`numcats'-1) {
            local Nbase = `Nconstraint'+1
            while `Nbase' <= `numcats' {
                * name of equation to constrain
                local catnm : word `Nconstraint' of `catnms'
                constraint define 999 [`catnm']

if `debug'>1 {
    di _new "#12"
    constraint dir
    di "Nconstraint and Nbase: `Nconstraint' `Nbase'"
    di "categories  : ``Nconstraint'' ``Nbase''"
    di "catvals: `catvals'"
    di "catnms:  `catnms'"
    di "mlogit `depvar' `rhsnam' `wtis' if `sample' == 1, base(``Nbase'') constr(999)"
}

                qui mlogit `depvar' `rhsnam' `wtis' ///
                  if `sample' == 1, base(``Nbase'') constr(999)
                qui lrtest . _LRbase
                scalar `chisq' = r(chi2)
                scalar `df' = r(df)
                scalar `pval' = r(p)
                local numrow = `numrow' + 1
                local s1`numrow' : word `Nconstraint' of `catnms8'
                local s2`numrow' : word `Nbase' of `catnms8'
                if (`pval'==.) { 
                    scalar `pval' = -9999 
                }
                mat `nxtrow' = ``Nconstraint'', ``Nbase'', `chisq', `df', `pval'
                mat roweq `nxtrow' = "`s1`numrow''"
                mat rownames `nxtrow' = "`s2`numrow''"
                mat `matlrc' = nullmat(`matlrc') \ `nxtrow'
                local Nbase = `Nbase' + 1
            }
            local Nconstraint = `Nconstraint' + 1
        } // while `Nconstraint' <= `numcats'

        mat colnames `matlrc' = cat1 cat2 chi2 df p
        local countto = rowsof(`matlrc')
        local count = 1
        while `count' <= `countto' {
            scalar `chisq' = `matlrc'[`count', 3]
            scalar `df' = `matlrc'[`count', 4]
            scalar `pval' = `matlrc'[`count', 5]
            if (`pval'==-9999) { 
                scalar `pval' = . 
            }
            di `smcl' in y %8s "`s1`count''" _col(9) "-" %8s "`s2`count''" ///
              _col(20) in g "`vlin'" _col(21) %9.3f in y `chisq' ///
              _col(30) %5.0f `df' _col(39) %4.3f `pval'
            local count = `count' + 1
        }
        di `smcl' in g _dup(19) "`dash'" "`botT'" _dup(24) "`dash'"
        qui estimates restore _Orig
        return matrix lrcomb `matlrc'
    } // if "`dolrcomb'"=="yes"

    qui estimates restore _Orig

    // drop estimates stored in program
    capture estimates drop _Orig
    capture estimates drop _OrigNewBase
    capture estimates drop _Aux
    capture estimates drop _suest
    capture estimates drop _LRbase
end

* version 3.0.4 2012-07-27 timing check
    * LR tests at ten times slower seemingly due to speed of mlogit 
    * when run as version 11, version 10 is speedier again
* version 3.0.4 2012-07-27 iia fixes and clean code
    * iia results differ slightly in stata 11+ due to new convegence criteria
* version 3.0.3 2012-07-27 no changes
* version 3.0.2 2012-07-27 factor with suest; lrcomb factor fix
*    TODO: verify the suest iia test when factors; jsl added common to make it work
* version 3.0.1 2012-07-26 fixes for _perhs.ado for factors
* version 3.0.0 2012-06-02 08.21 not called
* version 1.7.5 2007-06-29 stata 10 fixes
* version 1.7.4  8Dec2006 - suest bug fix
* version 1.7.2 18Aug2006 - fix mlogit program name bug
* version 1.7.1 29Jun2006 - add suest as documented
* version 1.7.0 22Jun2006 - fix warning for hausman; use estimates restore; add suest (undocumented)
* version 1.6.9 09Aug2005 - change terminology for alternatives
